import os
import re
import gzip
import pickle
from lib.common.util import printer_list
from lib.common.genome.seq_base import FastaRecord, read_fasta
from api.common.genome.blast import blast_to_sqlite
from Bio.Phylo.BaseTree import Tree
from lib.common.fileIO import tsv_file_dict_parse
from lib.common.math.set import merge_same_element_set
from collections import OrderedDict
from lib.common.genome.genome_feature2 import Gene, GeneSet
from lib.common.os import multiprocess_running


def mclmatrix_parser(file_name):
    with open(file_name, 'r') as f:
        mclmatrix = ""
        matrix_flag = 0
        for each_line in f:
            # each_line = each_line.strip()
            if re.match('^begin$', each_line):
                matrix_flag = 1
                continue
            if matrix_flag == 1:
                mclmatrix = mclmatrix + each_line

        group_string_list = mclmatrix.split('$')
        group_dir = OrderedDict()
        for i in group_string_list:
            j = i.split()
            if j[0] == ')':
                continue
            group_dir[j[0]] = j[1:]
    return group_dir


def OG_tsv_file_parse(file_name):
    file_info = tsv_file_dict_parse(file_name)
    
    OG_dict = OrderedDict()
    for i in file_info:
        OG_id = file_info[i]['Orthogroup']
        OG_dict[OG_id] = {}
        for j in file_info[i]:
            if j == 'Orthogroup':
                continue
            if file_info[i][j]:
                OG_dict[OG_id][j] = file_info[i][j].split(", ")
            else:
                OG_dict[OG_id][j] = ""
    
    return OG_dict


def write_OG_tsv_file(OG_dict, output_file, species_list=None):
    if not species_list:
        species_list = list(OG_dict[list(OG_dict.keys())[0]].keys())

    with open(output_file, 'w') as f:
        f.write(printer_list(species_list,head='Orthogroup\t')+"\n")
    
        for OG_id in OG_dict:
            string_list = []
            for sp_id in species_list:
                string_list.append(printer_list(OG_dict[OG_id][sp_id],sep=', '))
            output_string = printer_list(string_list,head=OG_id+"\t")
            f.write(output_string+"\n")

    return output_file


def get_species_id_from_gene_id(gene_id):
    return gene_id.split('_')[0]


def orthofinder_dir_tree_parse():
    pass

class OrthoFinderResults(object):
    def __init__(self, orthofinder_work_dir):
        # input
        self.dir = orthofinder_work_dir

    def pase_file_and_dir(self, modify_dict={}):
        """
        in modify_dict you can change
        """
        # get cwd and relpath of dir
        self.reldir = os.path.relpath(self.dir)
        self.cwd = os.getcwd()

        # get important dir and file in orthofinder output
        self.orthogroup_work_dir = self.reldir + "/WorkingDirectory"
        self.species_ID_file = self.orthogroup_work_dir + "/SpeciesIDs.txt"
        self.sequence_ID_file = self.orthogroup_work_dir + "/SequenceIDs.txt"

        for file_and_dir_tmp in os.listdir(self.reldir):
            if re.match('Orthologues_(.*)', file_and_dir_tmp):
                self.orthologues_results_dir = self.reldir + "/" + file_and_dir_tmp

        if not hasattr(self, 'orthologues_results_dir'):
            self.orthologues_results_dir = "Not found"

        self.orthologues_work_dir = self.orthologues_results_dir + "/WorkingDirectory"
        # self.orthologues_pair_dir = self.orthologues_results_dir + "/Orthologues"
        self.orthologues_pair_dir = self.reldir + "/Orthologues"
        self.orthogroup_tree_dir = self.orthologues_work_dir + "/Trees_ids"
        self.orthogroup_aln_dir = self.orthologues_work_dir + "/Alignments_ids"
        self.orthogroup_seq_dir = self.orthologues_work_dir + "/Sequences_ids"
        self.orthogroup_recon_tree_dir = self.orthologues_results_dir + "/Recon_Gene_Trees"

        for new_attr in modify_dict:
            if hasattr(self, new_attr):
                setattr(self, new_attr, modify_dict[new_attr])

    # speci and seq info
    def get_name_info(self, hash_table=False):
        self.species_info = {}
        with open(self.species_ID_file, 'r') as f:
            for each_line in f:
                each_line = each_line.strip()
                if len(re.findall("^(\d+)\:\s+(.*).fasta$", each_line)) > 0:
                    rename_id, speci_id = re.findall("^(\d+)\:\s+(.*).fasta$", each_line)[0]
                elif len(re.findall("^(\d+)\:\s+(.*).fa$", each_line)) > 0:
                    rename_id, speci_id = re.findall("^(\d+)\:\s+(.*).fa$", each_line)[0]
                self.species_info[rename_id] = {'id': speci_id}

        self.sequences_info = {}
        for speci_id in self.species_info:
            self.sequences_info[speci_id] = {}
            with open(self.sequence_ID_file, 'r') as f:
                for each_line in f:
                    each_line = each_line.strip()
                    rename_id, raw_id = re.findall("^(\S+)\:\s+(.*)$", each_line)[0]
                    raw_short_id = re.search('^(\S+)', raw_id).group(1)
                    speci_id_tmp = get_species_id_from_gene_id(rename_id)
                    gene_tmp = Gene(id=rename_id, species=speci_id_tmp)
                    gene_tmp.old_name = {
                        "raw_full_id": raw_id,
                        "raw_short_id": raw_short_id
                    }
                    self.sequences_info[speci_id][gene_tmp.id] = gene_tmp

        if hash_table:
            hash_table_file = self.orthogroup_work_dir + "/hash_table.pyb"
            if not os.path.exists(hash_table_file):
                self.raw_full_id_hash = {}
                self.raw_short_id_hash = {}
                self.new_id_hash = {}
                for species_id in self.sequences_info:
                    for rename_id in self.sequences_info[species_id]:
                        self.raw_full_id_hash[
                            self.sequences_info[species_id][rename_id].old_name['raw_full_id']] = rename_id
                        self.raw_short_id_hash[
                            self.sequences_info[species_id][rename_id].old_name['raw_short_id']] = rename_id
                        self.new_id_hash[rename_id] = self.sequences_info[species_id][rename_id].old_name['raw_short_id']
                OUT = open(hash_table_file, 'wb')
                pickle.dump((self.raw_full_id_hash, self.raw_short_id_hash, self.new_id_hash), OUT)
                OUT.close()
            else:
                TEMP = open(hash_table_file, 'rb')
                self.raw_full_id_hash, self.raw_short_id_hash, self.new_id_hash = pickle.load(TEMP)
                TEMP.close()

    def get_OrthoGroups(self):
        # orthogroup
        for file_and_dir_tmp in os.listdir(self.orthogroup_work_dir):
            if re.match('(clusters_OrthoFinder_.*_id_pairs.txt)', file_and_dir_tmp):
                self.orthogroup_file = self.orthogroup_work_dir + "/" + file_and_dir_tmp

        group_dir = mclmatrix_parser(self.orthogroup_file)
        self.orthogroup_dir = {}
        for i in group_dir:
            gene_set_name = "OG%07d" % int(i)
            gene_set_list = []
            for j in group_dir[i]:
                gene_set_list.append(self.sequences_info[get_species_id_from_gene_id(j)][j])
            gene_set_tmp = GeneSet(gene_set_name, gene_set_list)
            gene_set_tmp.tree_file = self.orthogroup_tree_dir + "/" + gene_set_name + '_tree_id.txt'
            gene_set_tmp.aln_file = self.orthogroup_aln_dir + "/" + gene_set_name + '.fa'
            gene_set_tmp.seq_file = self.orthogroup_seq_dir + "/" + gene_set_name + '.fa'

            self.orthogroup_dir[gene_set_name] = gene_set_tmp

    def get_OG_hash(self):
        self.OG_hash = {}
        for OG_id in self.orthogroup_dir:
            gene_list = self.orthogroup_dir[OG_id].gene_list
            for gene_tmp in gene_list:
                self.OG_hash[gene_tmp.id] = OG_id

    def read_aa_fasta(self):
        # find rename fasta file
        self.blast_results_file = {}
        for file_and_dir_tmp in os.listdir(self.orthogroup_work_dir):
            if re.match('Species(.*)\.fa', file_and_dir_tmp):
                speci_id = re.findall('Species(.*)\.fa', file_and_dir_tmp)[0]
                self.species_info[speci_id]['file_path'] = self.orthogroup_work_dir + "/" + file_and_dir_tmp

        for speci_id in self.species_info:
            speci_aa_file = self.species_info[speci_id]['file_path']
            fasta_dict = read_fasta(speci_aa_file)[0]
            for fasta_record_id in fasta_dict:
                fasta_record = fasta_dict[fasta_record_id]
                self.sequences_info[speci_id][fasta_record.seqname].model_aa_seq = fasta_record.seq

    # def add_aa_fasta(self, aa_fasta_dir):
    #     self.aa_fasta_dir = os.path.relpath(aa_fasta_dir)
    #     for speci_id in self.species_info:
    #         speci_old_id = self.species_info[speci_id]['id']
    #         for file_and_dir_tmp in os.listdir(self.aa_fasta_dir):
    #             if speci_old_id + ".fasta" == file_and_dir_tmp:
    #                 self.species_info[speci_id]['raw_file_path'] = self.aa_fasta_dir + "/" + file_and_dir_tmp
    #                 break

    def blast_to_sqlite(self, force_redo=False, threads=10):
        # find blast file
        self.blast_results_file = {}
        for file_and_dir_tmp in os.listdir(self.orthogroup_work_dir):
            if re.match('Blast(\d+)_(\d+)\.txt\.gz', file_and_dir_tmp):
                speci_1, speci_2 = re.findall('Blast(\d+)_(\d+)\.txt\.gz', file_and_dir_tmp)[0]
                self.blast_results_file[(speci_1, speci_2)] = {
                    'outfmt6': self.orthogroup_work_dir + "/" + file_and_dir_tmp
                }

        args_list = []
        for speci_pair in self.blast_results_file:
            blast_file = self.blast_results_file[speci_pair]['outfmt6']
            db_file = re.sub('\.txt\.gz', '', blast_file) + ".db"
            self.blast_results_file[speci_pair]['db'] = db_file
            if not force_redo and os.path.exists(db_file):
                continue
            args_list.append((db_file, blast_file, None, None, 6, None, None, None, False, True))

        if args_list != []:
            multiprocess_running(blast_to_sqlite, args_list, threads)

    def get_Orthologues(self):

        if not hasattr(self, 'raw_short_id_hash'):
            raise AttributeError("Running get_name with hash_table firstly")

        raw_name_list = [i['id'] for i in self.species_info.values()]

        # read all csv file to get orthologues, first get in a dir with many list
        orthogroup_sort_tmp_dir = {}
        for speci_raw_id_i in raw_name_list:
            tmp_dir = self.orthologues_pair_dir + "/Orthologues_" + speci_raw_id_i
            for speci_raw_id_j in raw_name_list:
                if speci_raw_id_i == speci_raw_id_j:
                    continue
                # tmp_file = "%s/%s__v__%s.tsv" % (tmp_dir, speci_raw_id_i, speci_raw_id_j)
                if os.path.exists("%s/%s__v__%s.tsv" % (tmp_dir, speci_raw_id_i, speci_raw_id_j)):
                    tmp_file = "%s/%s__v__%s.tsv" % (tmp_dir, speci_raw_id_i, speci_raw_id_j)
                elif os.path.exists("%s/%s__v__%s.csv" % (tmp_dir, speci_raw_id_i, speci_raw_id_j)):
                    tmp_file = "%s/%s__v__%s.csv" % (tmp_dir, speci_raw_id_i, speci_raw_id_j)
                else:
                    raise EnvironmentError("failed to found orthologues files")

                tmp_info = tsv_file_dict_parse(tmp_file)
                for id_tmp in tmp_info:
                    OG_id = tmp_info[id_tmp]['Orthogroup']
                    tmp_list = tmp_info[id_tmp][speci_raw_id_i].split(", ") + tmp_info[id_tmp][
                        speci_raw_id_j].split(", ")
                    tmp_list_new_id = [self.raw_short_id_hash[i] for i in tmp_list]
                    if OG_id not in orthogroup_sort_tmp_dir:
                        orthogroup_sort_tmp_dir[OG_id] = []
                    orthogroup_sort_tmp_dir[OG_id].append(tmp_list_new_id)

        # merge list by same iter in a orthogroup

        self.orthologues_dir = {}
        for OG_id in orthogroup_sort_tmp_dir:
            orthogroup_sort_tmp_dir[OG_id] = merge_same_element_set(orthogroup_sort_tmp_dir[OG_id])
            num = 0
            for orthologues_list in sorted(orthogroup_sort_tmp_dir[OG_id], key=lambda x: len(x), reverse=True):
                gene_set_name = OG_id + "_" + str(num)
                num = num + 1
                gene_set_list = []
                for j in orthologues_list:
                    gene_set_list.append(self.sequences_info[get_species_id_from_gene_id(j)][j])
                gene_set_tmp = GeneSet(gene_set_name, gene_set_list)
                self.orthologues_dir[gene_set_name] = gene_set_tmp

    def get_OL_hash(self):
        self.OL_hash = {}
        for OL_id in self.orthologues_dir:
            gene_list = self.orthologues_dir[OL_id].gene_list
            for gene_tmp in gene_list:
                self.OL_hash[gene_tmp.id] = OL_id


def get_orthologues(orthologues_dir, output_file):

    species_list = []

    for i in os.listdir(orthologues_dir):
        mobj = re.match(r'^Orthologues_(\S+)$', i)
        if mobj:
            speci = mobj.groups()[0]
            species_list.append(speci)

    orthogroup_sort_tmp_dir = {}
    seq_species_dict = {}
    for speci in species_list:
        tmp_dir = orthologues_dir + "/Orthologues_" + speci
        for speci_p in species_list:
            if speci == speci_p:
                continue
            # tmp_file = "%s/%s__v__%s.tsv" % (tmp_dir, speci_raw_id_i, speci_raw_id_j)
            if os.path.exists("%s/%s__v__%s.tsv" % (tmp_dir, speci, speci_p)):
                tmp_file = "%s/%s__v__%s.tsv" % (tmp_dir, speci, speci_p)
            elif os.path.exists("%s/%s__v__%s.csv" % (tmp_dir, speci, speci_p)):
                tmp_file = "%s/%s__v__%s.csv" % (tmp_dir, speci, speci_p)
            else:
                raise EnvironmentError("failed to found orthologues files")

            tmp_info = tsv_file_dict_parse(tmp_file)
            for id_tmp in tmp_info:
                OG_id = tmp_info[id_tmp]['Orthogroup']
                tmp_list = tmp_info[id_tmp][speci].split(", ") + tmp_info[id_tmp][
                    speci_p].split(", ")
                if OG_id not in orthogroup_sort_tmp_dir:
                    orthogroup_sort_tmp_dir[OG_id] = []
                orthogroup_sort_tmp_dir[OG_id].append(tmp_list)

                if not speci in seq_species_dict:
                    seq_species_dict[speci] = []
                if not speci_p in seq_species_dict:
                    seq_species_dict[speci_p] = []
                seq_species_dict[speci].extend(tmp_info[id_tmp][speci].split(", "))
                seq_species_dict[speci_p].extend(tmp_info[id_tmp][speci_p].split(", "))
    
    for i in seq_species_dict:
        seq_species_dict[i] = list(set(seq_species_dict[i]))

    seq_species_hash = {}
    for speci in seq_species_dict:
        for seq in seq_species_dict[speci]:
            seq_species_hash[seq] = speci
    

    # merge list by same iter in a orthogroup

    orthologues_dict = {}
    for OG_id in orthogroup_sort_tmp_dir:
        orthogroup_sort_tmp_dir[OG_id] = merge_same_element_set(orthogroup_sort_tmp_dir[OG_id])
        num = 0
        for orthologues_list in sorted(orthogroup_sort_tmp_dir[OG_id], key=lambda x: len(x), reverse=True):
            gene_set_name = OG_id + "_" + str(num)
            num = num + 1
            gene_set_list = []
            for j in orthologues_list:
                gene_set_list.append(j)
            gene_set_tmp = GeneSet(gene_set_name, gene_set_list)
            orthologues_dict[gene_set_name] = gene_set_tmp

    # output tsv file

    with open(output_file,'w') as f:
        header = ['Orthologues'] + species_list
        header_string = printer_list(header)
        f.write(header_string+"\n")

        for ol_id in orthologues_dict:
            output_list = [ol_id]
            gene_set = orthologues_dict[ol_id]
            for speci in species_list:
                gene_list = [i  for i in gene_set.gene_list if seq_species_hash[i] == speci]
                gene_string = printer_list(gene_list, sep=', ')
                output_list.append(gene_string)
            f.write(printer_list(output_list)+"\n")


def tsv_to_gene_pair(orthologues_tsv, species1, species2=None, output_file=None, huge_gene_family_filter=None):
    tmp_info = tsv_file_dict_parse(orthologues_tsv)

    with open(output_file, 'w') as f:
        for ol_id in tmp_info:
            if species2:
                if tmp_info[ol_id][species1] and tmp_info[ol_id][species2]:

                    speices_1_gene = tmp_info[ol_id][species1].split(", ")
                    speices_2_gene = tmp_info[ol_id][species2].split(", ")

                    if len(speices_1_gene) == 0 or len(speices_2_gene) == 0:
                        continue

                    for i in speices_1_gene:
                        for j in speices_2_gene:
                            f.write("%s\t%s\n" % (i, j))
            else:
                if tmp_info[ol_id][species1]:
                    speices_1_gene = tmp_info[ol_id][species1].split(", ")
            
                    if len(speices_1_gene) == 0:
                        continue

                    if huge_gene_family_filter and len(speices_1_gene) > huge_gene_family_filter:
                        continue

                    for i in speices_1_gene:
                        for j in speices_1_gene:
                            if i == j:
                                continue

                            f.write("%s\t%s\n" % (i, j))



if __name__ == '__main__':
    # parse orthofinder dir
    ortho_obj = OrthoFinderResults('/lustre/home/xuyuxing/Work/Csp/orthofinder/protein_seq/Results_Apr10')

    # get file and dir info
    ortho_obj.pase_file_and_dir(modify_dict={})

    # If you need to specify parameters you can use modify
    ortho_obj.pase_file_and_dir(modify_dict={'orthogroup_tree_dir': '/other/path'})

    # get speci and seq name info
    ortho_obj.get_name_info(hash_table=False)

    # get OrthoGroups info
    ortho_obj.get_OrthoGroups()

    # get orthologues
    ortho_obj.get_name_info(hash_table=True)
    ortho_obj.get_Orthologues()

    # change blast results to sqlite, can be dry_run just get exist db file path
    ortho_obj.blast_to_sqlite(force_redo=True)

    # # get raw seq id and changed seq id map
    # ortho_obj.get_RenameMap()

    # get orthogroups info from orthofinder dir
    ortho_obj.get_OrthoGroups()

    # get Orthologues info from orthofinder dir
    ortho_obj.get_Orthologues()

    # not by class

    # get Orthologues info from orthofinder dir
    orthologues_dir = '/lustre/home/xuyuxing/Work/Gel/mcscanx/Gel_vs_Aof/orthofinder2/OrthoFinder/Results_Nov21/Orthologues'
    output_file = '/lustre/home/xuyuxing/Work/Gel/mcscanx/Gel_vs_Aof/orthofinder2/OrthoFinder/Results_Nov21/Orthologues/Orthologues.tsv'
    get_orthologues(orthologues_dir, output_file)

    # get Orthologues pair between two species
    orthologues_tsv = '/lustre/home/xuyuxing/Work/Gel/mcscanx/Gel_vs_Other/orthofinder2/OrthoFinder/Results_Nov21/Orthologues/Orthologues.tsv'
    species1 = 'Aof.merge'
    species2 = 'Gel.merge'
    output_file = '/lustre/home/xuyuxing/Work/Gel/mcscanx/Gel_vs_Other/Gel_vs_Aof/Aof_vs_Gel.OL.pair'
    tsv_to_gene_pair(orthologues_tsv, species1, species2, output_file)






import numpy as np
from interlap import InterLap
from collections import OrderedDict
import networkx as nx
from itertools import combinations

from lib.common.math.interval import *
from lib.common.math.set import *


def interval_length_sum(interval_list, int_flag=True):
    """get length sum of a list of interval"""
    length = 0

    for i in interval_list:
        if int_flag:
            length += (abs(i[1] - i[0]) + 1)
        else:
            length += abs(i[1] - i[0])

    return length


def merge_all_dimension(input_matrix):
    record_num, dimension, range_num = input_matrix.shape

    if not range_num == 2:
        raise ValueError("interval range should have 2 number")

    output_array = []
    for i in range(dimension):
        output_array.append(
            (input_matrix[:, i, :].min(), input_matrix[:, i, :].max()))

    output_array = np.array(output_array)

    return output_array


def common_intervals(input_matrix, int_flag):
    def one_step(input_matrix, int_flag):
        record_num, dimension, range_num = input_matrix.shape

        if not range_num == 2:
            raise ValueError("interval range should have 2 number")

        find_inter = 0
        for i in range(record_num):
            for j in range(record_num):
                if i == j:
                    continue
                I_array = input_matrix[i]
                J_array = input_matrix[j]
                inter_flag = 1
                for k in range(dimension):
                    flag, deta = section(I_array[k], J_array[k], int_flag)
                    if not flag:
                        inter_flag = 0
                if inter_flag == 1:
                    new_array = merge_all_dimension(
                        np.array([I_array, J_array]))
                    if not new_array.shape == (dimension, range_num):
                        raise ValueError("merged array shape error, maybe bug")
                    new_array = np.array([new_array])
                    find_inter = 1
                    break
            if find_inter == 1:
                break

        if find_inter == 1:
            new_matrix = np.delete(input_matrix, (i, j), 0)
            new_matrix = np.concatenate((new_matrix, new_array), axis=0)
            return find_inter, new_matrix
        else:
            return find_inter, input_matrix

    matrix = input_matrix
    find_inter = 1
    while (find_inter):
        find_inter, matrix = one_step(matrix, int_flag)

    return matrix


def uniqify(seq, idfun=None):
    """
    a function that will delete redundancy elements
    note: slow than list(set(*))
    :param seq:
    :param idfun:
    :return:
    """
    # order preserving
    if idfun is None:
        def idfun(x): return x
    seen = {}
    result = []
    for item in seq:
        marker = idfun(item)
        # in old Python versions:
        # if seen.has_key(marker)
        # but in new ones:
        if marker in seen:
            continue
        seen[marker] = 1
        result.append(item)
    return result


def overlap_with_interlap_set(target_intervals, interlap_set, int_flag=False):
    """
    target_intervals = (50,100)
    interlap_set = InterLap([(1,70),(90,120)])

    overlap is (50,70) and (90,100), when int_flag is True
    no_overlap is (71, 89) = 19

    no_overlap ratio = 19/51 = 0.37
    """

    no_overlap = interval_minus_set(
        target_intervals, list(interlap_set.find(target_intervals)))
    overlap = interval_minus_set(target_intervals, no_overlap)
    overlap_length = sum_interval_length(overlap, close_range=int_flag)

    if int_flag:
        overlap_ratio = overlap_length / \
            (max(target_intervals) - min(target_intervals) + 1)
    else:
        overlap_ratio = overlap_length / \
            (max(target_intervals) - min(target_intervals))

    return overlap_ratio, overlap_length, overlap


def site_cluster(site_list, max_gap):
    """
    a function that will find site cluster which def by max_gap on one dimensional space
    :param site_list: if on interval_mode, input should be a list of tuples
                      e.g. intervals = [(1,5),(33,35),(40,33),(10,15),(13,18),(28,23),(70,80),(22,25),(38,50),(40,60)]
                      else:
                      input should be a list of int
                      e.g. list = [1,23,4,32,34]
    :return: merged list
    """

    sorted_list = sorted(list(set(site_list)))
    cluster_list = []
    temp_cluster = []
    for temp in sorted_list:
        if not temp_cluster:
            temp_cluster.append(temp)
        else:
            lower = temp_cluster[-1]
            if temp - lower <= max_gap:
                temp_cluster.append(temp)
            else:
                cluster_list.append(tuple(temp_cluster))
                temp_cluster = []
                temp_cluster.append(temp)
    cluster_list.append(tuple(temp_cluster))
    return cluster_list


def jaccord_index(set1, set2):
    set1 = set(set1)
    set2 = set(set2)

    return (len(set1 & set2) / len(set1 | set2))


def merge_order_list(one_list, two_list):
    """merge two number list"""

    for item in two_list:
        # 先区分元素是否比列表的最大元素还要大。
        if item < one_list[-1]:
            for i in range(len(one_list)):
                # 先比较，再插入
                if item <= one_list[i]:
                    one_list.insert(i, item)
                    break
        else:
            one_list.append(item)
    return one_list


def similarity_mcl_based_cluster():
    """
    similarity cluster, just good for clean gap like 0.2 to 0.8
    """

    pass



import subprocess
import os
import uuid
import shutil
import time
from multiprocessing import Pool, TimeoutError
from multiprocessing.dummy import Pool as ThreadPool
from functools import partial

from lib.common.os import *
from lib.common.util import *

__author__ = 'Yuxing Xu'


# time
def TimeStampToTime(timestamp):
    timeStruct = time.localtime(timestamp)
    return time.strftime('%Y-%m-%d %H:%M:%S', timeStruct)


# file and dir


def merge_file(input_file_list, output_file):
    with open(output_file, 'w') as f:
        for input_file in input_file_list:
            fr = open(input_file, 'r').read()
            if len(fr) > 0 and fr[-1] != '\n':
                fr = fr+'\n'
            f.write(fr)


def get_FileAccessTime(filePath):
    t = os.path.getatime(filePath)
    return TimeStampToTime(t)


def get_FileCreateTime(filePath):
    t = os.path.getctime(filePath)
    return TimeStampToTime(t)


def get_FileModifyTime(filePath):
    t = os.path.getmtime(filePath)
    return TimeStampToTime(t)


def print_tar_gz_info(tar_file):
    import tarfile
    tar = tarfile.open(tar_file, "r:gz")
    for tarinfo in tar:
        print(tarinfo.name, "is", tarinfo.size,
              "bytes in size and is ", end="")
        if tarinfo.isreg():
            print("a regular file.")
        elif tarinfo.isdir():
            print("a directory.")
        else:
            print("something else.")
    tar.close()


# running bash


def cmd_run_rltime(cmd_string, stdout_file, stderr_file, cwd=None):
    with open(stdout_file, 'w') as stdout_f:
        with open(stderr_file, 'w') as stderr_f:
            returncode = subprocess.call(
                cmd_string, cwd=cwd, shell=True, stdout=stdout_f, stderr=stderr_f)
    return returncode


def cmd_open(cmd_string, cwd=None):
    """
    open file by a shell cmd, such as zcat or samtools view
    :param cmd_string:
    :param cwd:
    :return:
    """
    p = subprocess.Popen(cmd_string, shell=True,
                         stderr=subprocess.PIPE, stdout=subprocess.PIPE, cwd=cwd)
    done = 0
    while not done:
        line_tmp = p.stdout.readline().decode()
        yield line_tmp
        if line_tmp == "":
            done = 1


# def multiprocess_running(f, args_list, pool_num, slience=True):
#     """
#
#     :param f: function name
#     :param args_list: a list include every args in a tuple
#     :param pool_num: process num of running at sametime
#     :param slience:
#     :return:
#     """
#
#     if pool_num == 0:
#         p = Pool()
#     else:
#         p = Pool(pool_num)
#     p_dict = {}
#     num = 0
#
#     for i in args_list:
#         p_dict['ID_' + str(num)] = {
#             'args': i,
#             'output': None,
#             'success': None,
#             'p_object': p.apply_async(f, args=i)
#         }
#         num = num + 1
#
#     if not slience:
#         print('Waiting for all subprocesses done...')
#     p.close()
#     p.join()
#     if not slience:
#         print('All subprocesses done.')
#
#     for i in p_dict:
#         p_dict[i]['output'] = p_dict[i]['p_object'].get()
#         p_dict[i]['success'] = p_dict[i]['p_object']._success
#         del p_dict[i]['p_object']
#
#     return p_dict


# def multiprocess_running(f, args_list, pool_num, log_file=None, silence=True, args_id_list=None, timeout=None):
#     """
#     :param f: function name
#     :param args_list: a list include every args in a tuple
#     :param pool_num: process num of running at sametime
#     :param silence:
#     :return:

#     example

#     def f(x, y):
#         time.sleep(1)
#         return x * y

#     args_list = list(zip(range(1,200),range(2,201)))
#     pool_num = 5

#     multiprocess_running(f, args_list, pool_num, log_file='/lustre/home/xuyuxing/Work/Other/saif/Meth/tmp/log')

#     """

#     num_tasks = len(list(args_list))

#     module_log = logging_init(f.__name__, log_file)
#     if not silence:
#         print('args_list have %d object and pool_num is %d' %
#               (num_tasks, pool_num))
#     module_log.info('running with mulitprocess')
#     module_log.info('args_list have %d object and pool_num is %d' %
#                     (num_tasks, pool_num))

#     p_dict = {}

#     if len(args_list) > 0:

#         f_with_para = partial(get_more_para, f)
#         abortable_func = partial(abortable_worker, f_with_para, timeout=timeout)

#         #    def get_more_para(para_tuple):
#         #        return f(*para_tuple)

#         #    args_list_unziped = zip(*args_list)

#         start_time = time.time()

#         if not silence:
#             print('Begin: ')
#         module_log.info('Begin: ')

#         with Pool(processes=pool_num, maxtasksperchild=1) as pool:
#             # for i, output in enumerate(pool.imap_unordered(f_with_para, args_list, chunksize=1)):
#             it = pool.imap(abortable_func, args_list, chunksize=1)
#             i=-1
#             while 1:
#                 i+=1
#                 # print(i)

#                 try:
#                     output = it.next()

#                     if args_id_list:
#                         job_id = args_id_list[i]
#                     else:
#                         job_id = 'ID_' + str(i)

#                     if output == "Aborting due to timeout":
#                         p_dict[job_id] = {
#                             'args': args_list[i],
#                             'output': None,
#                             'error': "timeout"
#                         }
#                     else:
#                         p_dict[job_id] = {
#                             'args': args_list[i],
#                             'output': output,
#                             'error': None
#                         }

#                     # print(i)
#                     round_time = time.time()
#                     if round_time - start_time > 5:
#                         if not silence:
#                             print('%d/%d %.2f%% parsed' %
#                                 (i, num_tasks, i / num_tasks * 100))
#                         module_log.info('%d/%d %.2f%% parsed' %
#                                         (i, num_tasks, i / num_tasks * 100))
#                         start_time = round_time

#                     # module_log.info('%d/%d %.2f%% parsed' % (i, num_tasks, i / num_tasks * 100))

#                 except StopIteration:
#                     break


#         module_log.info('%d/%d %.2f%% parsed' %
#                         (i, num_tasks, i / num_tasks * 100))
#         module_log.info('All args_list task finished')

#         if not silence:
#             print('%d/%d %.2f%% parsed' % (i, num_tasks, i / num_tasks * 100))
#             print('All args_list task finished')

#     del module_log.handlers[:]

#     return p_dict


if __name__ == '__main__':
    from multiprocessing import Queue

    share_data = [1, 2, 3, 4, 5, 6, 7]

    share_data_queue = Queue()

    from time import sleep

    def f(x):
        sleep(x)
        return x

    args_list = [(1,), (10,), (2,), (3,), (4,), (5,), (6,), (7,), (8,)]

    args_id_list = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i']

    cmd_result = multiprocess_running(
        f, args_list, 3, args_id_list=args_id_list, timeout=5)



import sys

sys.path.append("/lustre/home/xuyuxing/python_project/Genome_work_tools/")
from BCBio import GFF
from api.xuyuxing.file_parser.fileIO import dict_to_tsv_file
from lib.common.os import md5sum_maker
import re
import gzip
import os
import sys


def get_location_string(location_range_list, strand):
    location_range_list = sorted(location_range_list, key=lambda x: x[0])
    if len(location_range_list) > 1:
        location_string = ""
        for i in location_range_list:
            if min(i) == max(i):
                location_string = location_string + "%d," % min(i)
            else:
                location_string = location_string + "%d..%d," % (min(i), max(i))
        location_string = location_string.rstrip(",")
        location_string = "join(%s)" % location_string

    else:
        i = location_range_list[0]
        location_string = "%d..%d" % (i[0], i[1])

    if strand == -1:
        location_string = "complement(%s)" % location_string

    return location_string


def NCBIGffModel(in_file):
    # remove line with "part" tag, which GFF reader can't parse
    f_orgin = gzip.open(in_file, 'rt')
    f_filterd = open(in_file.replace(".gff.gz", "") + ".filterd.gff", "w")
    for row in f_orgin:
        row_filterd = re.match(r".*;part=.*", row)
        if not row_filterd:
            f_filterd.write(row)
    f_orgin.close()
    f_filterd.close()

    # NCBI gff file have many diff type
    ## best type, which have "gene_biotype" tag, it can tell me this gene is good
    in_handle = open(in_file.replace(".gff.gz", "") + ".filterd.gff")
    gene_dict = {}
    num = 0
    gene_model = {}
    for rec in GFF.parse(in_handle):
        for gene in rec.features:
            gene_dict[gene.id] = gene, rec.id
            if 'gene_biotype' in gene.qualifiers and gene.qualifiers['gene_biotype'][0] == 'protein_coding':
                protein_dict = {}
                for rna in gene.sub_features:
                    location_range_list = []
                    gene_length = 0
                    for cds in rna.sub_features:
                        if cds.type == "CDS":
                            if not "protein_id" in cds.qualifiers:
                                continue
                            # protein_id = cds.qualifiers["protein_id"][0]
                            cds_length = abs(cds.location.start - cds.location.end)
                            location_range_list.append((int(cds.location.start) + 1, int(cds.location.end)))
                            gene_length = gene_length + cds_length
                            # if protein_id not in protein_dict:
                            #     protein_dict[protein_id] = cds_length
                            # else:
                            #     protein_dict[protein_id] = protein_dict[protein_id] + cds_length

                    if len(location_range_list) == 0:
                        continue

                    location_string = get_location_string(location_range_list, gene.strand)

                    protein_dict[location_string] = gene_length

                if len(protein_dict) == 0:
                    # some time cds not under a mRNA, but close to gene
                    location_range_list = []
                    gene_length = 0
                    for cds in gene.sub_features:
                        if cds.type == "CDS":
                            if not "protein_id" in cds.qualifiers:
                                continue
                            # protein_id = cds.qualifiers["protein_id"][0]
                            location_range_list.append((int(cds.location.start) + 1, int(cds.location.end)))
                            cds_length = abs(cds.location.start - cds.location.end)
                            gene_length = gene_length + cds_length
                            # if protein_id not in protein_dict:
                            #     protein_dict[protein_id] = cds_length
                            # else:
                            #     protein_dict[protein_id] = protein_dict[protein_id] + cds_length

                    if len(location_range_list) == 0:
                        continue

                    location_string = get_location_string(location_range_list, gene.strand)

                    protein_dict[location_string] = gene_length

                if len(protein_dict) == 0:
                    continue

                model_protein = sorted(protein_dict, key=lambda x: protein_dict[x], reverse=True)[0]
                gene_model[gene.id] = (model_protein, rec.id)
    in_handle.close()

    ## this type don't have "gene_biotype" tag, we need try every gene
    if len(gene_model) == 0:  # maybe no pseudogene
        in_handle = open(in_file.replace(".gff.gz", "") + ".filterd.gff")
        gene_dict = {}
        num = 0
        gene_model = {}
        for rec in GFF.parse(in_handle):
            for gene in rec.features:
                gene_dict[gene.id] = gene, rec.id
                protein_dict = {}
                for rna in gene.sub_features:
                    location_range_list = []
                    gene_length = 0
                    for cds in rna.sub_features:
                        if cds.type == "CDS":
                            if not "protein_id" in cds.qualifiers:
                                continue
                            # protein_id = cds.qualifiers["protein_id"][0]
                            cds_length = abs(cds.location.start - cds.location.end)
                            location_range_list.append((int(cds.location.start) + 1, int(cds.location.end)))
                            gene_length = gene_length + cds_length
                            # if protein_id not in protein_dict:
                            #     protein_dict[protein_id] = cds_length
                            # else:
                            #     protein_dict[protein_id] = protein_dict[protein_id] + cds_length

                    if len(location_range_list) == 0:
                        continue

                    location_string = get_location_string(location_range_list, gene.strand)

                    protein_dict[location_string] = gene_length

                if len(protein_dict) == 0:
                    # some time cds not under a mRNA, but close to gene
                    location_range_list = []
                    gene_length = 0
                    for cds in gene.sub_features:
                        if cds.type == "CDS":
                            if not "protein_id" in cds.qualifiers:
                                continue
                            # protein_id = cds.qualifiers["protein_id"][0]
                            location_range_list.append((int(cds.location.start) + 1, int(cds.location.end)))
                            cds_length = abs(cds.location.start - cds.location.end)
                            gene_length = gene_length + cds_length
                            # if protein_id not in protein_dict:
                            #     protein_dict[protein_id] = cds_length
                            # else:
                            #     protein_dict[protein_id] = protein_dict[protein_id] + cds_length

                    if len(location_range_list) == 0:
                        continue

                    location_string = get_location_string(location_range_list, gene.strand)

                    protein_dict[location_string] = gene_length

                if len(protein_dict) == 0:
                    continue

                model_protein = sorted(protein_dict, key=lambda x: protein_dict[x], reverse=True)[0]
                gene_model[gene.id] = (model_protein, rec.id)
        in_handle.close()

    ## this type all gff file just have cds type

    if len(gene_model) == 0:  # maybe is all cds gff
        # raise ValueError(in_file)
        in_handle = open(in_file.replace(".gff.gz", "") + ".filterd.gff")
        gene_dict = {}
        num = 0
        gene_model = {}
        type_list = []

        for rec in GFF.parse(in_handle):
            location_range_dict = {}
            for cds in rec.features:
                gene_dict[cds.id] = cds, rec.id
                if cds.type == "sequence_alteration":
                    continue
                type_list.append(cds.type)
                if cds.type == "CDS":
                    if not "protein_id" in cds.qualifiers:
                        continue
                    protein_id = cds.qualifiers["protein_id"][0]
                    if not protein_id in location_range_dict:
                        location_range_dict[protein_id] = [[], cds.strand]
                    location_range_dict[protein_id][0].append((int(cds.location.start) + 1, int(cds.location.end)))

            for protein_id in location_range_dict:
                location_string = get_location_string(location_range_dict[protein_id][0],
                                                      location_range_dict[protein_id][1])
                gene_model[protein_id] = (location_string, rec.id)

        type_list = list(set(type_list))
        if not "CDS" in type_list:
            raise ValueError("Unknown gff style!")
        in_handle.close()

    os.remove(in_file.replace(".gff.gz", "") + ".filterd.gff")

    # # gene id in ncbi gff hard to link to pt cds fasta id, so we need change to locus_tag or Dbxref and so on
    #
    # gene_tmp_id = list(gene_model.keys())[0]
    # gene_tmp, chr_id = gene_dict[gene_tmp_id]
    #
    # if 'locus_tag' in gene_tmp.qualifiers:
    #     renew_id_flag = 'locus_tag'
    # elif 'Dbxref' in gene_tmp.qualifiers:
    #     renew_id_flag = 'Dbxref'
    # else:
    #     raise ValueError("I don't know what can I use as new gene id")

    # gene id in ncbi gff hard to link to pt cds fasta id, so we need change to locus_tag or Dbxref and so on
    # not each gene have locus_tag or Dbxref, but they have locus info, I will use it

    output_dict = {}
    for i in gene_model:
        location_string, chr_id = gene_model[i]

        start = min([int(i) for i in re.findall(r'\d+', location_string)])
        end = max([int(i) for i in re.findall(r'\d+', location_string)])

        if re.match(r'complement', location_string):
            strand = -1
        else:
            strand = 1

        output_dict[i] = {
            "location_string": location_string,
            "scaffold": chr_id,
            "start": start,
            "end": end,
            "strand": strand,
            # "locus_tag": gene.qualifiers['locus_tag'][0]
        }

    return output_dict


def JGIGffModel(in_file, out_file):
    import re

    def hash_map(dict_list):
        dict_hash = {}
        for i in dict_list:
            for j in dict_list[i]:
                if not j in dict_hash:
                    dict_hash[j] = []
                dict_hash[j].append(i)
        return dict_hash

    def integrate_element(dict_list, num):
        dict_hash = hash_map(dict_list)
        flag = 0
        deleted = []
        for i in dict_hash:
            if not len(dict_hash[i]) > 1:
                continue
            #        print i
            dict_list[num] = []
            for j in dict_hash[i]:
                if j in deleted:
                    continue
                dict_list[num] = list(set(dict_list[num] + dict_list[j]))
                del dict_list[j]
                deleted.append(j)
                flag = 1
            num = num + 1
            break
        return flag, num

    def NoRed_element(list_all):
        dict_list = {}
        num = 0
        for i in list_all:
            dict_list[num] = i
            num = num + 1
        # print dict_list
        flag = 1
        while flag == 1:
            flag, num = integrate_element(dict_list, num)
        #        print len(dict_list)
        output = []
        for i in dict_list:
            output.append(dict_list[i])
        return output

    def section(list1, list2):
        all = sorted(list(list(list1) + list(list2)))
        deta = all[2] - all[1] + 1
        smallest_length = min(abs(list1[1] - list1[0] + 1), abs(list2[1] - list2[0] + 1))
        if max(list1) >= min(list2) and max(list2) >= min(list1):
            If_inter = 1  # Yes
        else:
            If_inter = 0  # No
        overlap = float(deta) / float(smallest_length)
        return int(If_inter), overlap

    def Detect_Red_Range(all_list, overlap_cut=0.6):
        Candi_list = []
        for i in all_list:
            i_section = all_list[i][0]
            for j in all_list:
                j_section = all_list[j][0]
                if i == j:
                    continue
                # print i,j
                If_inter, overlap = section(i_section, j_section)
                if If_inter:
                    if overlap >= overlap_cut:
                        Candi_list.append(sorted([i, j]))
        return NoRed_element(Candi_list)

    F3 = gzip.open(in_file, 'rt')
    output = {}
    cds_info_dict = {}
    temp_id = 0
    for each_line in F3:
        each_line = re.sub('\n', '', each_line)
        info = each_line.split("\t")
        scaffold = info[0]
        type = info[2]
        if not type == "CDS":
            continue
        start = min(int(info[3]), int(info[4]))
        end = max(int(info[3]), int(info[4]))
        if info[6] == "+":
            strand = 1
        else:
            strand = -1
        qualifiers = info[8]
        qualifiers = qualifiers.split("; ")
        for i in qualifiers:
            name_matchObj = re.match(r'name \"(.*)\"', i)
            proteinId_matchObj = re.match(r'proteinId (\d+)', i)
            if name_matchObj:
                name = name_matchObj.group(1)
            if proteinId_matchObj:
                proteinId = proteinId_matchObj.group(1)

        temp_id = temp_id + 1
        cds_info = [proteinId, scaffold, start, end, strand, name]
        cds_info_dict[temp_id] = cds_info

        if not proteinId in output:
            output[proteinId] = [[proteinId, scaffold, start, end, strand, name], [temp_id]]
        else:
            if start < output[proteinId][0][2]:
                output[proteinId][0][2] = start
            if end > output[proteinId][0][3]:
                output[proteinId][0][3] = end
            output[proteinId][1].append(temp_id)
    F3.close()

    # mapping to scaffold
    mapping = {}
    for proteinId in output:
        [proteinId, scaffold, start, end, strand, name], cds_id = output[proteinId]
        if not scaffold in mapping:
            mapping[scaffold] = {}
            mapping[scaffold][1] = {}
            mapping[scaffold][-1] = {}
        mapping[scaffold][strand][proteinId] = [[start, end], cds_id]

    # No isoform
    Red_gene_list = []
    for scaffold in mapping:
        Red_gene_list = Red_gene_list + Detect_Red_Range(mapping[scaffold][1])
        Red_gene_list = Red_gene_list + Detect_Red_Range(mapping[scaffold][-1])

    smaller_proteins_list = []
    for pairs in Red_gene_list:
        id_length = []
        for proteins in pairs:
            total_length = 0
            for cds in output[proteins][1]:
                proteinId_all, scaffold, start, end, strand, name = cds_info_dict[cds]
                length = end - start + 1
                total_length = total_length + length
            id_length.append([proteins, total_length])
        id_length = sorted(id_length, key=lambda e: e[1], reverse=True)

        for i in id_length:
            if id_length.index(i) == 0:
                continue
            smaller_proteins_list.append(i[0])

    column_name_list = ["gene_id", "protein_id", "scaffold", "start", "end", "strand"]

    gene_dict_num = 0
    gene_model = {}
    for i in output:
        proteinId, scaffold, start, end, strand, name = output[i][0]
        if proteinId in smaller_proteins_list:
            continue
        gene_dict_num = gene_dict_num + 1

        gene_model[name] = {
            "gene_id": name,
            "protein_id": proteinId,
            "scaffold": scaffold,
            "start": start,
            "end": end,
            "strand": strand
        }

    column_name_list = ["gene_id", "protein_id", "scaffold", "start", "end", "strand"]

    dict_to_tsv_file(gene_model, out_file, column_name_list, seq=",", gzip_flag=True)

    md5_string = md5sum_maker(out_file)

    return gene_model, md5_string


def BIGGffModel(in_file):
    # remove line with "part" tag, which GFF reader can't parse
    in_handle = gzip.open(in_file, 'rt')
    gene_dict = {}
    num = 0
    gene_model = {}
    for rec in GFF.parse(in_handle):
        for gene in rec.features:
            gene_dict[gene.id] = gene, rec.id

            protein_dict = {}
            for cds in gene.sub_features:
                if not cds.type == "CDS":
                    continue
                protein_id = cds.qualifiers["Protein_Accession"][0]
                if not protein_id in protein_dict:
                    protein_dict[protein_id] = 0
                cds_length = abs(cds.location.start - cds.location.end)
                protein_dict[protein_id] = protein_dict[protein_id] + cds_length

            if len(protein_dict) == 0:
                protein_dict = {}
                for rna in gene.sub_features:
                    if not rna.type == "mRNA":
                        continue
                    for cds in rna.sub_features:
                        if not cds.type == "CDS":
                            continue
                        protein_id = cds.qualifiers["Protein_Accession"][0]
                        if not protein_id in protein_dict:
                            protein_dict[protein_id] = 0
                        cds_length = abs(cds.location.start - cds.location.end)
                        protein_dict[protein_id] = protein_dict[protein_id] + cds_length

            model_protein = sorted(protein_dict, key=lambda x: protein_dict[x], reverse=True)[0]
            gene_model[gene.id] = model_protein

    in_handle.close()

    output_dict = {}
    for i in gene_model:
        gene, rec_id = gene_dict[i]
        start = min(int(gene.location.start+1),int(gene.location.end))
        end = max(int(gene.location.start + 1), int(gene.location.end))


        output_dict[i] = {
            "protein_id": gene_model[i],
            "scaffold": rec_id,
            "start": start,
            "end": end,
            "strand": gene.strand,
            # "locus_tag": gene.qualifiers['locus_tag'][0]
        }

    return output_dict



if __name__ == '__main__':
    in_file = sys.argv[1]
    NCBIGffModel(in_file)



from api.common.genome.blast import outfmt6_read_big
from lib.xuyuxing.evolution.taxonomy import build_taxon_database, blast_taxon_assign, get_common_tree
from lib.common.os import multiprocess_running


def DiamondTaxonAssign_main(args):
    taxon_dict = build_taxon_database(args.tax_dir)

    args_list = []
    args_id_list = []
    for query_record in outfmt6_read_big(args.blast_results,
                                            fieldname=["qseqid", "sseqid", "staxids", "pident", "length",
                                                    "mismatch",
                                                    "gapopen", "qstart", "qend", "sstart", "send", "evalue",
                                                    "bitscore"]):

        query_name = query_record.qDef

        taxon_score_list = []
        for hit_tmp in query_record.hit:
            for taxon in hit_tmp.Hit_taxon_id:
                if taxon == '':
                    continue
                if taxon is None:
                    continue
                score = hit_tmp.hsp[0].Hsp_bit_score
                taxon_score_list.append((taxon, score))

        taxon_id_list = list(set([i[0] for i in taxon_score_list]))

        if taxon_id_list == []:
            continue

        common_tree = get_common_tree(taxon_id_list, taxon_dict)

        # root_taxon, fisher_taxon, q_value = blast_taxon_assign(taxon_score_list, taxon_dict, True)

        args_list.append((taxon_score_list, common_tree))
        args_id_list.append(query_name)

    cmd_result = multiprocess_running(blast_taxon_assign, args_list, args.num_threads, args_id_list=args_id_list)

    with open(args.output_file, 'w') as f:
        for query_id in args_id_list:
            if cmd_result[query_id]['output'] == "bug":
                print(query_id)
            else:
                root_taxon, fisher_taxon, q_value = cmd_result[query_id]['output']
                f.write("%s\t%s\t%s\t%s\t%s\t%.5f\n" % (
                    query_id, root_taxon, taxon_dict[root_taxon].sci_name, fisher_taxon,
                    taxon_dict[fisher_taxon].sci_name, q_value))



import re
import skbio
import random
import math
from copy import deepcopy
from pyfaidx import Fasta
import pandas as pd
from collections import OrderedDict
from interlap import InterLap
from BCBio import GFF
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio.SeqFeature import SeqFeature, FeatureLocation
from lib.common.math.interval import merge_intervals, interval_minus_set
from lib.common.genome.seq_base import read_fasta_by_faidx
from lib.common.os import cmd_run
from lib.common.fileIO import tsv_file_dict_parse
from lib.common.util import printer_list

# tools for seq as string

nucl_char_list = ["A", "T", "C", "G", "U"]
nucl_degenerate_char_list = ["M", "S", "W", "B", "D", "R", "H", "Y", "V", "K"]
any_nucl_char_list = ["N"]
prot_char_list = ["A", "P", "B", "Q", "C", "R", "D", "S", "E", "T", "F", "U", "G", "V", "H", "W", "I", "Y", "K", "Z", "L",
                  "M", "N"]
any_prot_char_list = ["X"]
stop_site_char_list = ["*"]
gap_site_char_list = ['-']


def sequence_clean(seq, seq_type='nucl', degenerate_sites_allowed=False, gap_allowed=False, translation_stop_allowed=False):
    # make code
    if seq_type == 'nucl':
        seq_type_flag = 0b0001
    elif seq_type == 'prot':
        seq_type_flag = 0b0000

    dsa_flag = degenerate_sites_allowed
    if dsa_flag:
        dsa_flag = 0b0010

    ga_flag = gap_allowed
    if ga_flag:
        ga_flag = 0b0100

    tsa_flag = translation_stop_allowed
    if tsa_flag:
        tsa_flag = 0b1000

    bit_field = seq_type_flag + dsa_flag + ga_flag + tsa_flag

    if bit_field == 0b0111:
        all_allowed = nucl_char_list + any_nucl_char_list + \
            nucl_degenerate_char_list + gap_site_char_list
        safe_remove = []
        safe_replace = any_nucl_char_list
    elif bit_field == 0b0011:
        all_allowed = nucl_char_list + any_nucl_char_list + nucl_degenerate_char_list
        safe_remove = gap_site_char_list
        safe_replace = any_nucl_char_list
    elif bit_field == 0b0101:
        all_allowed = nucl_char_list + any_nucl_char_list + gap_site_char_list
        safe_remove = []
        safe_replace = any_nucl_char_list
    elif bit_field == 0b0001:
        all_allowed = nucl_char_list + any_nucl_char_list
        safe_remove = gap_site_char_list
        safe_replace = any_nucl_char_list
    elif bit_field == 0b0000:
        all_allowed = prot_char_list + any_prot_char_list
        safe_remove = gap_site_char_list + stop_site_char_list
        safe_replace = any_prot_char_list
    elif bit_field == 0b0100:
        all_allowed = prot_char_list + any_prot_char_list + gap_site_char_list
        safe_remove = stop_site_char_list
        safe_replace = any_prot_char_list
    elif bit_field == 0b1100:
        all_allowed = prot_char_list + any_prot_char_list + \
            gap_site_char_list + stop_site_char_list
        safe_remove = []
        safe_replace = any_prot_char_list
    elif bit_field == 0b1000:
        all_allowed = prot_char_list + any_prot_char_list + stop_site_char_list
        safe_remove = gap_site_char_list
        safe_replace = any_prot_char_list
    else:
        raise ValueError("Bad input flag")

    new_seq = seq
    char_site = set(new_seq)
    bad_flag = False
    for i in char_site:
        if i in all_allowed:
            continue
        elif i in safe_remove:
            if i == "*":
                new_seq = re.sub("\*", "", new_seq)
            else:
                new_seq = re.sub(i, "", new_seq)
        else:
            if i == "*":
                new_seq = re.sub("\*", "", new_seq)
            else:
                new_seq = re.sub(i, safe_replace[0], new_seq)
            bad_flag = True

    return new_seq, bad_flag


def slice_scaffold_by_N(sequence, scaffold_maker="N"):
    """
    some scaffold sequence have many N in the seq, I can find the start and end for each Non-N seq, e.g. contig. I will
    return a list which have tuple(start,end) 0-base
    """
    subseqs = sequence.split(scaffold_maker)
    sub_seq_slice = []
    pointer = 0
    for i in subseqs:
        if len(i) == 0:
            pointer = pointer + 1
            continue
        sub_seq_slice.append((pointer, pointer + len(i)))
        pointer = pointer + len(i) + 1
    return sub_seq_slice


def contig_cutting(sequence, step, length, pointer_start=1):
    """
    cutting a sequence as windows, you can give me step and window length. I will return a generator which every one is
    start point and window sequence. pointer_start is the begin pointer it just for ID, will not change sequence cutting
    """
    seq_len = len(sequence)
    sub_seq_dict = {}
    for i in range(pointer_start - 1, seq_len, step):
        if seq_len - i > length:
            sub_seq = sequence[i:i + length]
            yield (pointer_start, sub_seq)
            sub_seq_dict[pointer_start] = sub_seq
        else:
            sub_seq = sequence[i:seq_len]
            yield (pointer_start, sub_seq)
            break
        pointer_start = pointer_start + step


def scaffold_cutting(sequence, step=500, length=1000, Consider_scaffold=True, shift_start=0):
    """
    cutting a sequence as windows, you can give me step and window length. I will return a generator which every one is
    start point and window sequence. pointer_start is the begin pointer it just for ID, will not change sequence cutting
    If the sequence is scaffold, I can split scaffold by N firstly, and get contig to make windows, the pointer will follow
    scaffold which based on 1
    """
    if Consider_scaffold is True:
        sub_seq_slice = slice_scaffold_by_N(sequence)
        # output_dict = {}
        for start, end in sub_seq_slice:
            sub_seq = sequence[start:end]
            for pointer, sub_sub_seq in contig_cutting(sub_seq, step, length, start + 1):
                yield (pointer, sub_sub_seq)
                # output_dict[pointer] = sub_sub_seq
    else:
        if shift_start != 0:
            # print(shift_start)
            yield (1, sequence[1 - 1:shift_start])

        # output_dict = {}
        for pointer, sub_seq in contig_cutting(sequence, step, length, shift_start + 1):
            yield (pointer, sub_seq)
            # output_dict[pointer] = sub_seq

    # return output_dict


def reverse_complement(seq, RNA=False):
    """
    #>>> seq = "TCGGinsGCCC"
    #>>> print "Reverse Complement:"
    #>>> print(reverse_complement(seq))
    #GGGCinsCCGA
    """
    alt_map = {'ins': '0'}
    if RNA:
        complement = {'A': 'U', 'C': 'G', 'G': 'C', 'U': 'A'}
    else:
        complement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A'}
    # for k, v in alt_map.iteritems():
    for k, v in alt_map.items():
        seq = seq.replace(k, v)
    bases = list(seq)
    bases = reversed([complement.get(base, base) for base in bases])
    bases = ''.join(bases)
    # for k, v in alt_map.iteritems():
    for k, v in alt_map.items():
        bases = bases.replace(v, k)
    return bases


def reverse_seq(seq):
    """just reverse"""
    return "".join(reversed(seq))


def complement_seq(seq):
    """just complement"""
    if 'U' in seq:
        complement = {'A': 'U', 'C': 'G', 'G': 'C', 'U': 'A'}
    else:
        complement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A'}
    bases = list(seq)
    bases = [complement.get(base, base) for base in bases]
    bases = ''.join(bases)
    return bases


def sub_seq(seq, site_start, site_end, strand, RNA):
    """get sub set of a seq"""
    start = min(site_start, site_end)
    end = max(site_start, site_end)
    sub_seq_out = seq[start - 1:end]

    if RNA is True:
        sub_seq_out = sub_seq_out.replace("T", "U")
    if strand == "-":
        sub_seq_out = reverse_complement(sub_seq_out, RNA)

    return sub_seq_out


def sequence_kmer_frequencies(sequence_input, kmer):
    s = skbio.Sequence(sequence_input)
    freqs = s.kmer_frequencies(kmer, overlap=False)
    return freqs


def sequence_entropy(sequence_input, kmer):
    freqs = sequence_kmer_frequencies(sequence_input, kmer)
    t_num = sum([freqs[i] for i in freqs])

    entropy = 0
    for i in freqs:
        p = freqs[i]/t_num
        entropy = entropy - (p * math.log(p, 2))

    return entropy


def random_sequence(seq_char, length, random_seed=None):
    """
    get random sequence
    :param seq_char: for DNA, you can give "ATCG"
    """
    random.seed(random_seed)
    sa = []
    for i in range(length):
        sa.append(random.choice(seq_char))
    salt = ''.join(sa)
    return salt


def get_seq_index_ignore_gap(give_site, seq_with_gap, seq_start=1, gap_chr='-'):
    """
    seq_with_gap = "------------ATGC---CAGTCA---ACG-GCATGCTA"
    give_site = 5
    seq_start = 1
    gap_chr = '-'

    give a seq with gap, and a site number, return index in gap seq
    """
    go_on_flag = True
    round_use_range = (0, 0)
    add_length = give_site - seq_start + 1
    round_true_site = 0
    while (go_on_flag):
        round_use_range = (
            round_use_range[1] + 1, round_use_range[1] + add_length)
        round_seq = seq_with_gap[round_use_range[0] - 1:round_use_range[1]]
        round_true_site = round_true_site + \
            len(round_seq) - round_seq.count(gap_chr)
        add_length = round_seq.count(gap_chr)
        print(round_use_range, round_seq, round_true_site, add_length)

        if add_length == 0:
            go_on_flag = False

    return round_use_range[1]


# tools for seq class

class BioSeq(object):
    def __init__(self, seq_id=None, seq='', seq_type='nucl'):
        self.id = seq_id
        self.seq = seq
        self.seq_type = seq_type

    def __str__(self):
        return self.seq

    def __len__(self):
        if len(self.seq) == 0 and hasattr(self, 'length'):
            return self.length
        else:
            return len(self.seq)

    def seq_clean(self, degenerate_sites_allowed=False, gap_allowed=False,
                  translation_stop_allowed=False):

        new_seq, bad_flag = sequence_clean(
            self.seq, self.seq_type, degenerate_sites_allowed, gap_allowed, translation_stop_allowed)
        self.seq = new_seq

    def seqs_upper(self):
        self.seq = self.seq.upper()

    def seqs_lower(self):
        self.seq = self.seq.lower()

    def no_wrap(self):
        self.seq = re.sub("\n", "", self.seq)

    def wrap(self, line_length=75):
        raw_seq = str(self.seq)
        raw_seq = re.sub("\n", "", raw_seq)
        new_seq = ""

        for j in (raw_seq[x: x + line_length] for x in range(0, len(raw_seq), line_length)):
            new_seq += j + "\n"

        self.seq = new_seq

        return new_seq

    def sub(self, site_start, site_end, strand, RNA):
        return sub_seq(self.seq, site_start, site_end, strand, RNA)

    def RNA(self):
        return self.seq.replace("T", "U")

    def reverse(self):
        return reverse_seq(self.seq)

    def complement(self):
        return complement_seq(self.seq)

    def reverse_complement(self):
        return complement_seq(reverse_seq(self.seq))


class FastaRecord(BioSeq):
    def __init__(self, seqname, seq=None, faidx=None):
        super(FastaRecord, self).__init__(seq)
        self.id = seqname
        self.faidx = faidx
        self.qualifiers = re.findall('\[([^\[\]]+)=([^\[\]]+)\]', seqname)

    def short_id(self):
        name_short = re.search('^(\S+)', self.id).group(1)
        return name_short

    @property
    def seq(self):
        if self.faidx is not None:
            return str(self.faidx)
        else:
            return self._seq

    @seq.setter
    def seq(self, value):
        self._seq = value

    def read_seq_to_mem(self):
        self._seq = str(self.faidx)

    def sub(self, site_start, site_end, strand, RNA):
        if self.faidx is not None:
            output = str(self.faidx[site_start - 1:site_end])
            if RNA is True:
                output = output.replace("T", "U")
            if strand == "-":
                output = reverse_complement(output, RNA)
            return output
        else:
            return sub_seq(self.seq, site_start, site_end, strand, RNA)

    def len(self):
        if hasattr(self, 'faidx'):
            return len(self.faidx)
        else:
            return len(self.seq)

    def get_GC_ratio(self):
        gc_now = (self.seq.count('C') + self.seq.count('G') +
                  self.seq.count('c') + self.seq.count('g'))/self.len()
        return gc_now


class FastqRecord(FastaRecord):
    def __init__(self, seqname, seq, quality):
        super(FastqRecord, self).__init__(seqname, seq)
        self.quality = quality


def faidx2xyx(pyfaidx_record, write_seq_mem):
    try:
        seq_name = pyfaidx_record.long_name
    except:
        seq_name = pyfaidx_record.name

    if write_seq_mem:
        return FastaRecord(seq_name, str(pyfaidx_record), pyfaidx_record)
    else:
        return FastaRecord(seq_name, None, pyfaidx_record)


def read_fasta_by_faidx(file_name, write_seq_mem=False, force_wrap_file=False):
    # if file_name.split(".")[-1] == 'gz':
    #     cmd_run("gzip %s" % file_name, slience=True)
    #     file_name = remove_file_name_suffix(file_name, 1)

    try:
        output_dict = OrderedDict()
        for i in Fasta(file_name):
            output_dict[i.name] = faidx2xyx(i, write_seq_mem)
        return output_dict
    except:
        try:
            cmd_run("samtools faidx %s" % file_name)
            output_dict = OrderedDict()
            for i in Fasta(file_name):
                output_dict[i.name] = faidx2xyx(i, write_seq_mem)
            return output_dict
        except:
            raise EnvironmentError("fasta file not suit for pyfaidx")

    #
    #
    # wrap_file_name = remove_file_name_suffix(file_name, 1) + ".pyfaidx.fasta"
    # if force_wrap_file:
    #     wrap_fasta_file(file_name, wrap_file_name, 75)
    #     try:
    #         return {i.name: faidx2xyx(i, write_seq_mem) for i in Fasta(wrap_file_name)}
    #     except:
    #         raise EnvironmentError("fasta file not suit for pyfaidx")
    # else:
    #     if os.path.exists(wrap_file_name):
    #         try:
    #             return {i.name: faidx2xyx(i, write_seq_mem) for i in Fasta(wrap_file_name)}
    #         except:
    #             wrap_fasta_file(file_name, wrap_file_name, 75)
    #             try:
    #                 return {i.name: faidx2xyx(i, write_seq_mem) for i in Fasta(wrap_file_name)}
    #             except:
    #                 raise EnvironmentError("fasta file not suit for pyfaidx")
    #     else:
    #         try:
    #             return {i.name: faidx2xyx(i, write_seq_mem) for i in Fasta(file_name)}
    #         except:
    #             wrap_fasta_file(file_name, wrap_file_name, 75)
    #             try:
    #                 return {i.name: faidx2xyx(i, write_seq_mem) for i in Fasta(wrap_file_name)}
    #             except:
    #                 raise EnvironmentError("fasta file not suit for pyfaidx")
    #


def write_fasta(fasta_record_list, output_file, wrap_length=75, upper=False):
    with open(output_file, 'w') as f:
        for record in fasta_record_list:
            record.no_wrap()
            raw_seq = str(record.seq)

            if upper:
                raw_seq = raw_seq.upper()

            f.write(">%s\n" % record.id)
            for x in range(0, len(raw_seq), wrap_length):
                i = raw_seq[x: x + wrap_length]
                f.write(i + "\n")

# class for genome


class ChrLoci(object):
    def __init__(self, chr_id=None, strand=None, start=None, end=None, species_id=None):
        self.chr_id = chr_id
        self.sp_id = species_id

        if strand is None:
            self.strand = strand
        elif strand == "+" or str(strand) == '1':
            self.strand = "+"
        elif strand == "-" or str(strand) == '-1':
            self.strand = "-"
        else:
            self.strand = None

        if end is not None and start is not None:
            self.start = min(int(start), int(end))
            self.end = max(int(start), int(end))
            self._range = (self.start, self.end)
            self.length = abs(self.end - self.start) + 1

    # def __eq__(self, other):
    #     """Implement equality by comparing all the location attributes."""
    #     if not isinstance(other, ChrLoci):
    #         return False
    #     return self.start == other.start and \
    #            self.end == other.end and \
    #            self.strand == other.strand and \
    #            self.chr_id == other.chr_id

    def get_fancy_name(self):
        if self.strand is None:
            self.fancy_name = "%s:%d-%d" % (self.chr_id, self.start, self.end)
        else:
            self.fancy_name = "%s:%d-%d:%s" % (self.chr_id,
                                               self.start, self.end, self.strand)
        return self.fancy_name

    def __str__(self):
        try:
            self.get_fancy_name()
            if self.sp_id:
                return "%s %s" % (self.sp_id, self.fancy_name)
            else:
                return self.fancy_name
        except:
            return "No detail range"

    __repr__ = __str__

    @property
    def range(self):
        return self._range

    @range.setter
    def range(self, value):
        start, end = value
        if end is not None and start is not None:
            self.start = min(int(start), int(end))
            self.end = max(int(start), int(end))
            self._range = (self.start, self.end)

    def get_sequence(self, data_fasta_file, RNA=False):
        sequence = read_fasta_by_faidx(data_fasta_file)[self.chr_id].sub(self._range[0], self._range[1], self.strand,
                                                                         RNA)
        self.get_fancy_name()
        return BioSeq(sequence, self.fancy_name)

    def get_sequence_quick(self, seq_dict, RNA=False):
        """
        seq_dict should load sequence into memory, and Bioseq not have faidx attr
        """
        sequence = seq_dict[self.chr_id].sub(
            self._range[0], self._range[1], self.strand, RNA)
        self.get_fancy_name()
        return BioSeq(sequence, self.fancy_name)

    def len(self):
        return abs(self.end - self.start) + 1

    def __eq__(self, other):
        return self.chr_id == other.chr_id and self.strand == other.strand and self.start == other.start and self.end == other.end and self.sp_id == other.sp_id

    def __hash__(self):
        return hash(id(self))


class GenomeFeature(ChrLoci):
    def __init__(self, id=None, type=None, chr_loci=None, qualifiers={}, sub_features=None):
        if chr_loci is None:
            super(GenomeFeature, self).__init__(
                chr_id=None, strand=None, start=None, end=None)
        else:
            super(GenomeFeature, self).__init__(chr_id=chr_loci.chr_id, strand=chr_loci.strand, start=chr_loci.start,
                                                end=chr_loci.end)
        self.id = id
        self.type = type
        self.chr_loci = chr_loci
        self.sub_features = sub_features
        self.qualifiers = qualifiers

    def sgf_len(self):
        self.sgf_len_dir = {i: 0 for i in list(
            set([sgf.type for sgf in self.sub_features]))}

        for sgf in self.sub_features:
            sgf_len = abs(sgf.start - sgf.end) + 1
            self.sgf_len_dir[sgf.type] += sgf_len

    def __eq__(self, other):
        return self.id == other.id and self.chr_loci == other.chr_loci and self.type == other.type

    def __hash__(self):
        return hash(id(self))


class mRNA(GenomeFeature):
    def __init__(self, id=None, chr_loci=None, qualifiers=None, sub_features=None, cds_seq=None, aa_seq=None,
                 cDNA_seq=None, from_gf=None):
        if not from_gf is None:
            super(mRNA, self).__init__(id=from_gf.id, type='mRNA', chr_loci=from_gf.chr_loci,
                                       qualifiers=from_gf.qualifiers, sub_features=from_gf.sub_features)
        else:
            super(mRNA, self).__init__(id=id, type='mRNA', chr_loci=chr_loci, qualifiers=qualifiers,
                                       sub_features=sub_features)
        self.cds_seq = cds_seq
        self.aa_seq = aa_seq
        self.cDNA_seq = cDNA_seq

    def build_mRNA_seq(self, genome_file):
        if not hasattr(self, 'sub_features'):
            raise ValueError("build need sub_features")

        self.cds_seq = feature_seq_extract(
            self.sub_features, genome_file, "CDS")
        self.cDNA_seq = feature_seq_extract(
            self.sub_features, genome_file, "exon")

        good_orf, phase, aa_seq = cds_judgment(self.cds_seq)

        self.aa_seq = aa_seq
        self.cds_phase = phase
        self.cds_good_orf = good_orf

    def get_introns(self):

        exon_list = [i for i in sub_gf_traveler(self) if i.type in [
            'exon', 'CDS', 'five_prime_UTR', 'three_prime_UTR', 'five_prime_utr', 'three_prime_utr']]
        exon_interval_list = sorted(merge_intervals(
            [(i.start, i.end) for i in exon_list]))
        min_s = min([i[0] for i in exon_interval_list] + [i[1]
                                                          for i in exon_interval_list])
        max_s = max([i[0] for i in exon_interval_list] + [i[1]
                                                          for i in exon_interval_list])
        intron_interval_list = interval_minus_set(
            (min_s, max_s), exon_interval_list)
        intron_interval_list = sorted(
            intron_interval_list, key=lambda x: x[0], reverse=(self.strand == '-'))

        intron_list = []
        num = 1
        for i in intron_interval_list:
            intron_list.append(GenomeFeature(id="%s.intron.%d" % (self.id, num), type='intron', chr_loci=ChrLoci(
                chr_id=self.chr_id, strand=self.strand, start=min(i), end=max(i), species_id=self.sp_id), qualifiers=None, sub_features=[]))
            num += 1

        self.sub_features.extend(intron_list)


class Gene(GenomeFeature):
    def __init__(self, id=None, species=None, chr_loci=None, qualifiers=None, sub_features=None, model_cds_seq=None,
                 model_aa_seq=None, model_cDNA_seq=None, from_gf=None, chr_rank=None):
        if not from_gf is None:
            super(Gene, self).__init__(id=from_gf.id, type='gene', chr_loci=from_gf.chr_loci,
                                       qualifiers=from_gf.qualifiers, sub_features=from_gf.sub_features)
        else:
            super(Gene, self).__init__(id=id, type='gene', chr_loci=chr_loci, qualifiers=qualifiers,
                                       sub_features=sub_features)
        self.model_cds_seq = model_cds_seq
        self.model_aa_seq = model_aa_seq
        self.model_cDNA_seq = model_cDNA_seq
        self.species = species
        self.chr_rank = chr_rank

    def build_gene_seq(self, genome_file=None):
        if not hasattr(self, 'sub_features'):
            raise ValueError("build need sub_features")

        all_mRNA_list = [mRNA(from_gf=gf) for gf in self.sub_features]

        # remove mRNA without CDS
        mRNA_list = []
        for mRNA_now in all_mRNA_list:
            cds_num = len(
                [i for i in mRNA_now.sub_features if i.type == 'CDS'])
            if cds_num > 0:
                mRNA_list.append(mRNA_now)

        # [i.build_mRNA_seq(genome_file) for i in mRNA_list]
        [i.sgf_len() for i in mRNA_list]

        longest_cds_mRNA = sorted(
            mRNA_list, key=lambda x: x.sgf_len_dir['CDS'], reverse=True)[0]
        self.model_mRNA_id = longest_cds_mRNA.id
        self.model_mRNA = longest_cds_mRNA

        if genome_file is not None:
            longest_cds_mRNA.build_mRNA_seq(genome_file)

            self.model_cds_seq = longest_cds_mRNA.cds_seq
            self.model_aa_seq = longest_cds_mRNA.aa_seq
            self.model_cDNA_seq = longest_cds_mRNA.cDNA_seq
            self.model_cds_good_orf = longest_cds_mRNA.cds_good_orf
            self.model_cds_phase = longest_cds_mRNA.cds_phase

        self.sub_features = mRNA_list

    def __str__(self):
        try:
            return "Gene: %s (%s)" % (self.id, self.chr_loci.get_fancy_name())
        except:
            return "Gene: %s" % self.id

    __repr__ = __str__


class Chromosome(BioSeq):
    def __init__(self, chr_id=None, species=None, version=None, seq='', length=None, feature_dict=None, qualifiers={}, auto_get_gene_index=True):
        # base info
        super(Chromosome, self).__init__(
            seq_id=chr_id, seq=seq, seq_type='nucl')
        self.sp_id = species
        self.version = version
        self.qualifiers = qualifiers

        if length:
            self.length = length

        # chromesome feature info
        if feature_dict and auto_get_gene_index:
            self.auto_get_gene_index(feature_dict)

    def auto_get_gene_index(self, feature_dict):
        self.load_genome_features(feature_dict)
        self.gene_index = self.build_index('gene')
        self.sync_species_and_gene_rank_info()

    def load_genome_features(self, feature_dict):
        chr_feature_dict = OrderedDict()
        for type_tmp in feature_dict:
            chr_feature_dict[type_tmp] = OrderedDict()
            for gf_tmp_id in feature_dict[type_tmp]:
                gf_tmp = feature_dict[type_tmp][gf_tmp_id]
                if gf_tmp.chr_loci.chr_id == self.id:
                    chr_feature_dict[type_tmp][gf_tmp.id] = gf_tmp
        self.feature_dict = chr_feature_dict

    def build_index(self, top_type='gene'):
        if not hasattr(self, 'feature_dict'):
            raise ValueError("build_index need feature_dict")

        feature_index = {
            "+": InterLap(),
            "-": InterLap(),
            ".": InterLap()
        }

        for type_tmp in self.feature_dict:
            for gf_tmp_id in self.feature_dict[type_tmp]:
                gf_tmp = self.feature_dict[type_tmp][gf_tmp_id]
                index_tmp = feature_index[gf_tmp.strand]
                index_tmp.add(
                    (gf_tmp.chr_loci.start, gf_tmp.chr_loci.end, gf_tmp))

                feature_index["."].add(
                    (gf_tmp.chr_loci.start, gf_tmp.chr_loci.end, gf_tmp))

        return feature_index

    def sync_species_and_gene_rank_info(self):
        num = 0
        for s, e, gf in self.gene_index['.']:
            gf.sp_id = self.sp_id
            gf.index = num
            num += 1


class Genome(object):
    def __init__(self, genome_id=None, species=None, version=None, chr_dict=None):
        self.id = genome_id
        self.sp_id = species
        self.version = version
        self.chr_dict = chr_dict

        if self.chr_dict:
            self.get_gene_dict()

    def get_gene_dict(self):
        self.gene_dict = {}
        for chr_id in self.chr_dict:
            chromo_now = self.chr_dict[chr_id]
            for gene_id in chromo_now.feature_dict['gene']:
                self.gene_dict[gene_id] = chromo_now.feature_dict['gene'][gene_id]


def build_genome(genome_id, species, genome_fasta_file=None, gff_file=None, version=None):
    chr_dict = OrderedDict()

    if genome_fasta_file:
        genome_seq_dict = read_fasta_by_faidx(genome_fasta_file)
        for contig_id in genome_seq_dict:
            contig_seq = genome_seq_dict[contig_id]
            chr_dict[contig_id] = Chromosome(chr_id=contig_id, species=species, version=version, seq='', length=len(
                contig_seq), feature_dict=None, qualifiers={})

    if gff_file:
        gff_dict = read_gff_file(gff_file)
        gff_by_chr_dict = {}
        for top_type in gff_dict:
            for gf_id in gff_dict[top_type]:
                gf = gff_dict[top_type][gf_id]
                if gf.chr_id not in gff_by_chr_dict:
                    gff_by_chr_dict[gf.chr_id] = {}
                if top_type not in gff_by_chr_dict[gf.chr_id]:
                    gff_by_chr_dict[gf.chr_id][top_type] = {}
                gff_by_chr_dict[gf.chr_id][top_type][gf_id] = gf

        if len(chr_dict) > 0:
            for chr_id in chr_dict:
                chromo_now = chr_dict[chr_id]
                chromo_now.auto_get_gene_index(gff_by_chr_dict[chr_id])
        else:
            for chr_id in gff_by_chr_dict:
                max_site = 0
                for top_type in gff_by_chr_dict[chr_id]:
                    for gf_id in gff_by_chr_dict[chr_id][top_type]:
                        gf = gff_by_chr_dict[chr_id][top_type][gf_id]
                        max_site = max(gf.start, gf.end, max_site)
                chr_dict[contig_id] = Chromosome(chr_id=chr_id, species=species, version=version,
                                                 seq='', length=max_site, feature_dict=gff_by_chr_dict[chr_id], qualifiers={})

    return Genome(genome_id=genome_id, species=species, version=version, chr_dict=chr_dict)


def ft2cl(feature_location, chr_id):
    """
    create ChrLoci by FeatureLocation from BCBio
    """
    return ChrLoci(chr_id=chr_id, strand=feature_location.strand, start=feature_location.start + 1,
                   end=feature_location.end)


def cl2ft(chr_loci):
    """
    create FeatureLocation from BCBio by ChrLoci
    """
    if chr_loci.strand == "+" or chr_loci.strand == 1:
        strand = 1
    elif chr_loci.strand == "-" or chr_loci.strand == -1:
        strand = -1
    else:
        strand = 0

    return FeatureLocation(chr_loci.start - 1, chr_loci.end, strand=strand)


def sf2gf(sf, chr_id):
    """
    create GenomeFeature by SeqFeature from BCBio
    """
    sf_cl = ft2cl(sf.location, chr_id)
    gf = GenomeFeature(id=sf.id, type=sf.type, chr_loci=sf_cl)
    gf.qualifiers = sf.qualifiers

    # parse sub_feature
    if hasattr(sf, 'sub_features') and len(sf.sub_features) != 0:
        gf.sub_features = []
        for sub_sf in sf.sub_features:
            gf.sub_features.append(sf2gf(sub_sf, chr_id))

    return gf


def gf2sf(gf, source=None):
    """
    create SeqFeature from BCBio by GenomeFeature
    """
    fl = cl2ft(gf.chr_loci)

    qualifiers = gf.qualifiers
    if source is None:
        if 'source' in gf.qualifiers:
            source = gf.qualifiers['source']
        else:
            source = '.'

    qualifiers["source"] = source
    qualifiers["ID"] = gf.id

    sf = SeqFeature(fl, type=gf.type, qualifiers=qualifiers)

    # parse sub_feature
    sf.sub_features = []
    if hasattr(gf, 'sub_features') and not gf.sub_features is None:
        for sub_gf in gf.sub_features:
            sf.sub_features.append(gf2sf(sub_gf, source))

    sf.id = gf.id
    sf.qualifiers["ID"] = gf.id

    sf = deepcopy(sf)

    return sf


def read_gff_file(gff_file):
    feature_dict = OrderedDict()

    with open(gff_file, 'r') as in_handle:
        for rec in GFF.parse(in_handle):
            for feature in rec.features:
                new_feature = sf2gf(feature, rec.id)
                if new_feature.type not in feature_dict:
                    feature_dict[new_feature.type] = OrderedDict()
                feature_dict[new_feature.type][new_feature.id] = new_feature

    return feature_dict


def write_gff_file(gf_list, output_gff_file, source=None, sort=False):
    rec_dict = OrderedDict()

    for gf in gf_list:
        if gf.chr_id not in rec_dict:
            rec = SeqRecord(Seq(""), gf.chr_id, description='')
            rec.features = []
            rec_dict[gf.chr_id] = rec
        sf = gf2sf(gf, source)
        sf = deepcopy(sf)
        rec_dict[gf.chr_id].features.append(sf)

    if sort:
        for ci in rec_dict:
            ft_list = rec_dict[ci].features
            ft_sorted_list = sorted(
                ft_list, key=lambda x: int(x.location.start))
            rec_dict[ci].features = ft_sorted_list

    rec_list = [rec_dict[i] for i in rec_dict]

    with open(output_gff_file, "a") as f:
        GFF.write(rec_list, f)

# class for compare genome


def if_conserved(species_list, conserved_arguments):
    """
    conserved_arguments = [
        [['Cau', 'Ini', 'Sly'], ['Ath', 'Aco']],
        [2,2]
    ]
    """
    sp_group_lol = conserved_arguments[0]
    group_min_num_list = conserved_arguments[1]

    conserved_flag = True

    for sp_group_list, min_num in zip(sp_group_lol, group_min_num_list):
        if len(set(species_list) & set(sp_group_list)) < min_num:
            conserved_flag = False

    return conserved_flag


class GeneSet(object):
    def __init__(self, id, gene_list):
        self.id = id
        self.gene_list = gene_list
        self.tree_file = None
        self.seq_file = None
        self.aln_file = None

    def speci_stat(self):
        output_dir = {}
        for i in self.gene_list:
            if i.species not in output_dir:
                output_dir[i.species] = 0
            output_dir[i.species] = output_dir[i.species] + 1
        return output_dir

    def __str__(self):
        return "ID: %s; Gene Num: %d" % (self.id, len(self.gene_list))


class OrthoGroup(GeneSet):
    def __init__(self, id=None, from_gene_list=None, species_list=None, from_OG_dict=None):
        if from_OG_dict and from_gene_list is None:
            if not species_list:
                species_list = list(from_OG_dict.keys())

            from_gene_list = []
            for sp_id in from_OG_dict:
                for g_id in from_OG_dict[sp_id]:
                    from_gene_list.append(Gene(id=g_id, species=sp_id))

        if species_list:
            gene_dict = {i: [] for i in species_list}
        else:
            gene_dict = {}
            for i in from_gene_list:
                gene_dict[i.species] = []

        gene_list = [i for i in from_gene_list if i.species in gene_dict]

        super(OrthoGroup, self).__init__(id=id, gene_list=gene_list)

        self.gene_dict = gene_dict
        for i in self.gene_list:
            self.gene_dict[i.species].append(i)

        self.species_stat = {}
        for sp_id in self.gene_dict:
            self.species_stat[sp_id] = len(self.gene_dict[sp_id])

        self.species_list = list(self.species_stat.keys())

    def conserved(self, conserved_arguments_dict):
        non_zero_species_list = [
            i for i in self.species_stat if self.species_stat[i] == 0]

        return if_conserved(non_zero_species_list, conserved_arguments_dict)


class OrthoGroups(object):
    def __init__(self, id=None, OG_tsv_file=None, from_OG_dict=None, from_OG_list=None, species_list=None, note=None, new_OG_name=False):
        self.id = id
        self.note = note
        self.OG_tsv_file = OG_tsv_file
        self.from_OG_dict = from_OG_dict

        if OG_tsv_file:
            OG_dict = self.build_from_OG_tsv_file(
                OG_tsv_file, species_list=species_list)
        elif from_OG_dict:
            OG_dict = self.build_from_OG_dict(
                from_OG_dict, species_list=species_list)
        elif from_OG_list:
            OG_dict = self.build_from_OG_list(
                from_OG_list, species_list=species_list, new_OG_name=new_OG_name)
        else:
            OG_dict = OrderedDict()

        self.OG_dict = OG_dict
        if species_list:
            self.species_list = species_list
        else:
            self.species_list = OG_dict[list(OG_dict.keys())[0]].species_list
        self.OG_id_list = list(self.OG_dict.keys())

    def get(self, key_tuple):
        """
        key_tuple: ('OG1234', 'Ath'), or ('OG1234', None), or (None, 'Ath')
        """

        OG_id, sp_id = key_tuple

        if OG_id and sp_id:
            return self.OG_dict[OG_id].gene_dict[sp_id]
        elif OG_id:
            return self.OG_dict[OG_id].gene_dict
        elif sp_id:
            output_dict = {}
            for i in self.OG_dict:
                output_dict[i] = self.OG_dict[i].gene_dict[sp_id]
            return output_dict

    def build_from_OG_tsv_file(self, OG_tsv_file, species_list=None):
        file_info = tsv_file_dict_parse(OG_tsv_file)

        full_species_list = list(
            file_info[list(file_info.keys())[0]].keys())[1:]

        OG_dict = OrderedDict()

        for i in file_info:
            OG_id = file_info[i]['Orthogroup']

            gene_list = []
            for j in file_info[i]:
                if j == 'Orthogroup':
                    continue
                if file_info[i][j] == '':
                    continue
                gene_list.extend([Gene(id=k, species=j)
                                  for k in file_info[i][j].split(", ")])

            if species_list:
                OG_dict[OG_id] = OrthoGroup(
                    id=OG_id, from_gene_list=gene_list, species_list=species_list)
            else:
                OG_dict[OG_id] = OrthoGroup(
                    id=OG_id, from_gene_list=gene_list, species_list=full_species_list)

        return OG_dict

    def build_from_OG_dict(self, OG_dict, species_list=None):
        """
        OG_dict = {
            "OG0001": {
                "Ath": ['AT0001', 'AT0002'],
                "Osa": ['OS0001', 'OS0002']
            }
        }
        """
        new_OG_dict = OrderedDict()

        for OG_id in OG_dict:

            if isinstance(OG_dict[OG_id], dict):
                gene_list = []
                for sp_id in OG_dict[OG_id]:
                    gene_list.extend([Gene(id=i, species=sp_id)
                                      for i in OG_dict[OG_id][sp_id]])
                new_OG_dict[OG_id] = OrthoGroup(
                    id=OG_id, from_gene_list=gene_list, species_list=species_list)
            else:
                new_OG_dict[OG_id] = OG_dict[OG_id]

        return new_OG_dict

    def build_from_OG_list(self, OG_list, species_list=None, new_OG_name=False):
        if species_list:
            all_species_list = species_list
        else:
            all_species_list = []
            for og in OG_list:
                all_species_list.extend(og.species_list)
            all_species_list = list(set(all_species_list))

        new_OG_dict = OrderedDict()

        num = 0
        for og in OG_list:
            if new_OG_name:
                og_id = "OG%d" % num
                num += 1
            else:
                og_id = og.id

            gene_list = [
                gene for gene in og.gene_list if gene.species in set(all_species_list)]
            new_OG_dict[og_id] = OrthoGroup(
                id=og_id, from_gene_list=gene_list, species_list=all_species_list)

        return new_OG_dict

    def write_OG_tsv_file(self, output_file, species_list=None):
        if not species_list:
            species_list = self.species_list

        with open(output_file, 'w') as f:
            f.write(printer_list(species_list, head='Orthogroup\t')+"\n")

            for OG_id in self.OG_dict:
                OG = self.OG_dict[OG_id]
                string_list = []
                for sp_id in species_list:
                    string_list.append(printer_list(
                        [i.id for i in OG.gene_dict[sp_id]], sep=', '))
                output_string = printer_list(string_list, head=OG_id+"\t")
                f.write(output_string+"\n")

        return output_file

    def get_conserved_OG_list(self, conserved_arguments_dict):

        output_list = []
        for i in self.OG_dict:
            og = self.OG_dict[i]
            if og.conserved():
                output_list.append(i)

        return output_list

    def remove_some_OG(self, removed_OG_list):
        for i in removed_OG_list:
            self.OG_id_list.remove(i)
            del self.OG_dict[i]


class Species(object):
    def __init__(self,
                 sp_id=None,
                 taxon_id=None,
                 species_name=None,
                 genome_file=None,
                 gff_file=None,
                 pt_file=None,
                 cDNA_file=None,
                 cds_file=None):

        self.sp_id = sp_id
        self.taxon_id = taxon_id
        self.species_name = species_name
        self.genome_file = genome_file
        self.gff_file = gff_file
        self.pt_file = pt_file
        self.cDNA_file = cDNA_file
        self.cds_file = cds_file


def read_species_info(species_info_table, add_option=[]):

    species_info_df = pd.read_excel(species_info_table)
    species_info_dict = {}
    for index in species_info_df.index:
        sp_id = str(species_info_df.loc[index]['sp_id'])

        base_info_dict = {i: None for i in [
            'taxon_id', 'species_name', 'genome_file', 'gff_file', 'pt_file', 'cDNA_file', 'cds_file']}

        for i in base_info_dict:
            if i in species_info_df.loc[index]:
                base_info_dict[i] = species_info_df.loc[index][i]

        species_info_dict[sp_id] = Species(
            sp_id, base_info_dict['taxon_id'], base_info_dict['species_name'], base_info_dict['genome_file'], base_info_dict['gff_file'], base_info_dict['pt_file'], base_info_dict['cDNA_file'], base_info_dict['cds_file'])

        for i in add_option:
            if i in species_info_df.loc[index]:
                setattr(species_info_dict[sp_id], i,
                        species_info_df.loc[index][i])

    return species_info_dict


class GenePair(object):
    def __init__(self, q_gene, s_gene, property_dict):
        self.q_gene = q_gene
        self.s_gene = s_gene
        self.property = property_dict

    def __str__(self):
        return "%s vs %s" % (self.q_gene, self.s_gene)

    def reverse_myself(self):
        new_GP = GenePair(self.s_gene, self.q_gene, self.property)
        return new_GP


class SyntenyBlock(object):
    def __init__(self, sb_id=None, q_chr=None, s_chr=None, strand=None, gene_pair_dict=None, property_dict={}, parameter_dict={}):
        self.id = sb_id
        self.property = property_dict
        self.parameter = parameter_dict
        self.strand = strand
        self.q_chr = q_chr
        self.s_chr = s_chr
        self.gene_pair_dict = gene_pair_dict

        if gene_pair_dict:
            self.get_full_info()

    def get_full_info(self):

        first_q_gene_loci = min(
            [self.gene_pair_dict[i].q_gene.loci for i in self.gene_pair_dict])
        last_q_gene_loci = max(
            [self.gene_pair_dict[i].q_gene.loci for i in self.gene_pair_dict])
        first_s_gene_loci = min(
            [self.gene_pair_dict[i].s_gene.loci for i in self.gene_pair_dict])
        last_s_gene_loci = max(
            [self.gene_pair_dict[i].s_gene.loci for i in self.gene_pair_dict])

        self.query_gene_list = []
        for i in range(first_q_gene_loci, last_q_gene_loci + 1):
            self.query_gene_list.append(q_genome.chr_dict[self.q_chr][i])

        self.subject_gene_list = []
        for i in range(first_s_gene_loci, last_s_gene_loci + 1):
            self.subject_gene_list.append(s_genome.chr_dict[self.s_chr][i])

        self.first_q_gene = q_genome.chr_dict[self.q_chr][min(
            [i.loci for i in self.query_gene_list])]
        self.last_q_gene = q_genome.chr_dict[self.q_chr][max(
            [i.loci for i in self.query_gene_list])]
        self.first_s_gene = s_genome.chr_dict[self.s_chr][min(
            [i.loci for i in self.subject_gene_list])]
        self.last_s_gene = s_genome.chr_dict[self.s_chr][max(
            [i.loci for i in self.subject_gene_list])]

        self.first_q_gene_loci = self.first_q_gene.loci
        self.last_q_gene_loci = self.last_q_gene.loci
        self.first_s_gene_loci = self.first_s_gene.loci
        self.last_s_gene_loci = self.last_s_gene.loci

        self.query_from = min([q_genome.gene_dict[self.first_q_gene.id].gf.start, q_genome.gene_dict[self.first_q_gene.id].gf.end,
                               q_genome.gene_dict[self.last_q_gene.id].gf.start, q_genome.gene_dict[self.last_q_gene.id].gf.end])
        self.query_to = max([q_genome.gene_dict[self.first_q_gene.id].gf.start, q_genome.gene_dict[self.first_q_gene.id].gf.end,
                             q_genome.gene_dict[self.last_q_gene.id].gf.start, q_genome.gene_dict[self.last_q_gene.id].gf.end])
        self.subject_from = min([s_genome.gene_dict[self.first_s_gene.id].gf.start, s_genome.gene_dict[self.first_s_gene.id].gf.end,
                                 s_genome.gene_dict[self.last_s_gene.id].gf.start, s_genome.gene_dict[self.last_s_gene.id].gf.end])
        self.subject_to = max([s_genome.gene_dict[self.first_s_gene.id].gf.start, s_genome.gene_dict[self.first_s_gene.id].gf.end,
                               s_genome.gene_dict[self.last_s_gene.id].gf.start, s_genome.gene_dict[self.last_s_gene.id].gf.end])

    def reverse_myself(self, new_sb_id):
        gene_pair_dict = {
            i: self.gene_pair_dict[i].reverse_myself() for i in self.gene_pair_dict}
        new_sb = SyntenyBlock(new_sb_id, self.q_sp, self.s_sp,
                              self.strand, gene_pair_dict, self.property, self.parameter)
        return new_sb

    def __str__(self):
        return "Q = %s:%s gene: %d-%d (%d) base: %d-%d (%d) vs S = %s:%s gene: %d-%d (%d) base: %d-%d (%d), %s, have %d gene pair" % (self.q_sp, self.q_chr, self.first_q_gene_loci, self.last_q_gene_loci, self.last_q_gene_loci - self.first_q_gene_loci + 1,  self.query_from, self.query_to, self.query_to - self.query_from + 1, self.s_sp, self.s_chr, self.first_s_gene_loci, self.last_s_gene_loci, self.last_s_gene_loci - self.first_s_gene_loci + 1,   self.subject_from, self.subject_to, self.subject_to - self.subject_from + 1, self.strand, len(self.gene_pair_dict))

    __repr__ = __str__


if __name__ == '__main__':
    from config import project_dir

    Ath_genome_fasta_file = project_dir + \
        "/example_dataset/genome/Ath/T3702N0.genome.fasta"
    Ath_genome_gff3 = project_dir + "/example_dataset/genome/Ath/T3702N0.genome.gff3"

    Ath_genome = build_genome(
        'Ath', 'Ath', genome_fasta_file=Ath_genome_fasta_file, gff_file=Ath_genome_gff3, version='xyx')



import re
import os
from lib.common.fileIO import tsv_file_dict_parse
from api.common.genome.blast import outfmt5_read
from lib.common.genome.seq_base import read_fasta_by_faidx
from lib.common.math.interval import merge_intervals
from lib.common.util import printer_list
from lib.common.os import mkdir, cmd_run
from config import taxon_nr_db, taxon_dump
from src.xuyuxing.tools.seqtools import FragmentGenome_main
from src.xuyuxing.tools.taxontools import DiamondTaxonAssign_main
from src.xuyuxing.GenomeTools.ContaminationFilter import ContaminationFilter_main


class args_tmp():
    pass    


script_dir_path = os.path.split(os.path.realpath(__file__))[0]

ath_chl_aa = script_dir_path+"/organelle/Ath/Ath.chl.aa.fasta"
ath_chl_rRNA = script_dir_path+"/organelle/Ath/Ath.chl.rRNA.fasta"
osa_chl_aa = script_dir_path+"/organelle/Osa/Osa.chl.aa.fasta"
osa_chl_rRNA = script_dir_path+"/organelle/Osa/Osa.chl.rRNA.fasta"

ath_mit_aa = script_dir_path+"/organelle/Ath/Ath.mit.aa.fasta"
ath_mit_rRNA = script_dir_path+"/organelle/Ath/Ath.mit.rRNA.fasta"
osa_mit_aa = script_dir_path+"/organelle/Osa/Osa.mit.aa.fasta"
osa_mit_rRNA = script_dir_path+"/organelle/Osa/Osa.mit.rRNA.fasta"

ath_nucl_rRNA = script_dir_path+"/Ath_rRNA/rRNA.gene.fa"

ref_seq = {
    'chl': {
        'rRNA': {
            'Ath': ath_chl_rRNA,
            'Osa': osa_chl_rRNA,
        },
        'aa': {
            'Ath': ath_chl_aa,
            'Osa': osa_chl_aa,
        },
    },
    'mit': {
        'rRNA': {
            'Ath': ath_mit_rRNA,
            'Osa': osa_mit_rRNA,
        },
        'aa': {
            'Ath': ath_mit_aa,
            'Osa': osa_mit_aa,
        },
    },
    'nuc': {
        'rRNA': {
            'Ath': ath_nucl_rRNA,
        }
    },
}


def get_contamination_report(genome_fasta, work_dir, threads=56, max_target_seqs=50):
    mkdir(work_dir, True)

    split_genome_fasta = work_dir + "/split.genome.fasta"
    diamond_output = work_dir + "/nr.diamond.bls"
    taxon_assign_output = work_dir + "/nr.diamond.taxon"
    contamination_report = work_dir + "/contamination_report.txt"

    # FragmentGenome
    args = args_tmp()
    args.genome_file = genome_fasta
    args.output_file = split_genome_fasta
    args.step = 10000
    args.length = 10000
    args.shift_start = 0
    args.Consider_scaffold = True
    args.entropy_threshold = 3

    FragmentGenome_main(args)

    # diamond
    cmd_string = "diamond blastx --query %s --max-target-seqs %d --db %s --evalue 1e-5 --out %s --outfmt 6 qseqid sseqid staxids pident length mismatch gapopen qstart qend sstart send evalue bitscore --threads %s" % (
        split_genome_fasta,
        max_target_seqs,
        taxon_nr_db,
        diamond_output,
        threads
    )
    cmd_run(cmd_string, None, 1, True)

    # DiamondTaxonAssign
    args = args_tmp()
    args.blast_results = diamond_output
    args.tax_dir = taxon_dump
    args.num_threads = threads
    args.output_file = taxon_assign_output

    DiamondTaxonAssign_main(args)

    # ContaminationFilter
    args = args_tmp()
    args.split_genome_fasta = split_genome_fasta
    args.taxon_assign_output = taxon_assign_output
    args.tax_dir = taxon_dump
    args.output_file = taxon_assign_output

    ContaminationFilter_main(args)


def read_assembly(assembly_file):
    """
    assembly_file = '/lustre/home/xuyuxing/Database/Gel/genome/assembly/contig_filter/Gel.genome.v1.0.final.assembly'
    """

    contig_dict = {}
    scaffold_dict = {}
    scaf_id = 0
    with open(assembly_file, 'r') as f:
        for each_line in f:
            each_line = each_line.strip()
            matchObj = re.match('^>(\S+) (\d+) (\d+)$', each_line)
            if matchObj:
                contig_name, ID_num, contig_length = matchObj.groups()

                contig_name = re.sub(r':::debris', '', contig_name)
                contig_name = re.sub(r':::', '_', contig_name)

                contig_dict[ID_num] = (contig_name, ID_num, contig_length)
            else:
                scaf_info = each_line.split()

                scaf_contig_id_list = []
                scaf_contig_length_sum = 0
                for contig_id in scaf_info:
                    contig_id = re.sub(r'-', '', contig_id)
                    scaf_contig_id_list.append(contig_dict[contig_id][0])
                    scaf_contig_length_sum += int(contig_dict[contig_id][2])

                scaffold_dict['scaf_%d' % scaf_id] = {
                    "contig_list": scaf_contig_id_list,
                    "contig_length": scaf_contig_length_sum
                }
                scaf_id += 1

    return scaffold_dict


def filter_polt():
    pass

    """
    % matplotlib inline
    import matplotlib
    import numpy as np
    import matplotlib.pyplot as plt

    avg_depth_list = [float(genome_depth_dict[i]['avg_depth']) for i in genome_depth_dict]

    from lib.common.fileIO import tsv_file_dict_parse
    from api.common.genome.blast import outfmt6_read

    org_bls_dict = outfmt6_read(args.organelle_blast)

    genome_depth_dict = tsv_file_dict_parse(args.genome_seq_depth)

    avg_depth_list = [float(genome_depth_dict[i]['avg_depth']) for i in genome_depth_dict]

    import matplotlib
    import matplotlib.pyplot as plt
    import numpy as np
    import matplotlib.patches as patches
    import matplotlib.path as path

    fig = plt.figure()
    ax = fig.add_subplot(111)

    # Fixing random state for reproducibility
    # np.random.seed(19680801)

    # histogram our data with numpy
    # data = np.random.randn(1000)
    n, bins = np.histogram(avg_depth_list, bins='auto')

    # get the corners of the rectangles for the histogram
    left = np.array(bins[:-1])
    right = np.array(bins[1:])
    bottom = np.zeros(len(left))
    top = bottom + n
    nrects = len(left)

    nverts = nrects * (1 + 3 + 1)
    verts = np.zeros((nverts, 2))
    codes = np.ones(nverts, int) * path.Path.LINETO
    codes[0::5] = path.Path.MOVETO
    codes[4::5] = path.Path.CLOSEPOLY
    verts[0::5, 0] = left
    verts[0::5, 1] = bottom
    verts[1::5, 0] = left
    verts[1::5, 1] = top
    verts[2::5, 0] = right
    verts[2::5, 1] = top
    verts[3::5, 0] = right
    verts[3::5, 1] = bottom

    barpath = path.Path(verts, codes)
    # patch = patches.PathPatch(barpath, facecolor='green', edgecolor='yellow', alpha=0.5)
    patch = patches.PathPatch(barpath, facecolor='blue', edgecolor="blue", linewidth=0.0)
    ax.add_patch(patch)

    ax.set_xlim(left[0], right[-1])
    ax.set_ylim(bottom.min(), top.max())

    plt.show()"""


def ContigFilter_main(args):

    genome_fasta = read_fasta_by_faidx(args.genome_file)
    contig_list = list(genome_fasta.keys())

    # GSS depth
    info_dict = tsv_file_dict_parse(args.genome_seq_depth)
    genome_depth_dict = {}

    for i in info_dict:
        genome_depth_dict[info_dict[i]['contig']
                          ] = float(info_dict[i]['avg_depth'])

    # GC ratio
    gc_dict = {}
    for contig_id in contig_list:
        gc_now = (genome_fasta[contig_id].seq.count('C') + genome_fasta[contig_id].seq.count('G') +
                  genome_fasta[contig_id].seq.count('c') + genome_fasta[contig_id].seq.count('g'))/genome_fasta[contig_id].len()
        gc_dict[contig_id] = gc_now

    # Contig in Hi-C scaffold
    scaffold_dict = read_assembly(args.hic_assembly)
    contig_in_huge_scaffold = []
    for scaf_id in scaffold_dict:
        if scaffold_dict[scaf_id]['contig_length'] > 1000000:
            contig_in_huge_scaffold.extend(
                scaffold_dict[scaf_id]['contig_list'])

    # rRNA hit
    rRNA_bls_dict = outfmt5_read(args.rRNA_blast)

    rRNA_identity_dict = {}

    for query_id in ['18S', '25S']:
        rRNA_identity_dict[query_id] = {}
        query_record = rRNA_bls_dict[query_id]
        for hit_record in query_record.hit:
            hit_max_identity = 0
            for hsp_record in hit_record.hsp:
                identity = hsp_record.Hsp_identity / int(query_record.qLen)
                if identity > hit_max_identity:
                    hit_max_identity = identity
            rRNA_identity_dict[query_id][hit_record.Hit_def] = hit_max_identity

    contig_rRNA_ident_dict = {}
    for contig_id in contig_list:
        info_tuple = []
        for rRNA_id in ['18S', '25S']:
            if contig_id in rRNA_identity_dict[rRNA_id]:
                info_tuple.append(rRNA_identity_dict[rRNA_id][contig_id])
            else:
                info_tuple.append(0.0)
        contig_rRNA_ident_dict[contig_id] = tuple(info_tuple)

    # Organelle
    org_bls_dict = outfmt5_read(args.organelle_blast)

    subject_contig_dict = {}
    for query_id in org_bls_dict:
        query_record = org_bls_dict[query_id]
        for hit_record in query_record.hit:
            if hit_record.Hit_def not in subject_contig_dict:
                subject_contig_dict[hit_record.Hit_def] = {
                    "all_range": [],
                    "hit_list": [],
                    "contig_length": 0
                }
            for hsp_record in hit_record.hsp:
                subject_contig_dict[hit_record.Hit_def]["all_range"].append(
                    (hsp_record.Hsp_hit_from, hsp_record.Hsp_hit_to))
            subject_contig_dict[hit_record.Hit_def]["hit_list"].append(
                hit_record)
            subject_contig_dict[hit_record.Hit_def]["contig_length"] = int(
                hit_record.Hit_len)
            subject_contig_dict[hit_record.Hit_def]["contig_ratio"] = int(
                hit_record.Hit_len)

    for contig in subject_contig_dict:
        subject_contig_dict[contig]["all_range"] = merge_intervals(
            subject_contig_dict[contig]["all_range"], True)
        subject_range_length = 0
        for i in subject_contig_dict[contig]["all_range"]:
            subject_range_length += max(i) - min(i) + 1
        subject_contig_dict[contig]["subject_range_length"] = subject_range_length
        subject_contig_dict[contig]["contig_ratio"] = subject_range_length / subject_contig_dict[contig][
            "contig_length"]

        subject_contig_dict[contig]["query_info"] = {}
        for hit_record in subject_contig_dict[contig]["hit_list"]:
            query_range = []
            for hsp_record in hit_record.hsp:
                query_range.append(
                    (hsp_record.Hsp_query_from, hsp_record.Hsp_query_to))
            query_range = merge_intervals(query_range)

            range_length = 0
            for i in query_range:
                range_length += max(i) - min(i) + 1

            short_qDef = re.search('^(\S+)', hit_record.query.qDef).group(1)
            subject_contig_dict[contig]["query_info"][short_qDef] = {
                'query_range': query_range,
                'query_length': int(hit_record.query.qLen),
                'query_range_length': int(range_length),
                'query_ratio': int(range_length) / int(hit_record.query.qLen),
            }

    organelle_judge_dict = {}
    organelle_query_info = {}

    for contig_id in contig_list:
        if contig_id in subject_contig_dict and subject_contig_dict[contig_id]['contig_length'] < 1000000 and subject_contig_dict[contig_id]["contig_ratio"] > 0.5 and not (contig_id in contig_in_huge_scaffold) and genome_depth_dict[contig_id] > 100:
            organelle_judge_dict[contig_id] = True

            query_info = []
            query_id_list = sorted(subject_contig_dict[contig_id]['query_info'],
                                   key=lambda x: subject_contig_dict[contig_id]['query_info'][x]['query_ratio'],
                                   reverse=True)

            for query_id in query_id_list:
                query_length = subject_contig_dict[contig_id]['query_info'][query_id]["query_length"]
                query_range_length = subject_contig_dict[contig_id]['query_info'][query_id][
                    "query_range_length"]
                query_ratio = subject_contig_dict[contig_id]['query_info'][query_id]["query_ratio"]
                string_tmp = "%s:%d/%d(%.2f)" % (query_id,
                                                 query_range_length, query_length, query_ratio)
                query_info.append(string_tmp)

            query_info_str = printer_list(query_info, sep=",")

            organelle_query_info[contig_id] = query_info_str

        else:
            organelle_judge_dict[contig_id] = False

    # Contamination hit

    # note

        # is a organelle

    # write report
    with open(args.output_file, 'w') as f:
        header_list = ["Contig_id", "Contig_length", "GSS_depth", "GC_content",
                       "In_HiC_huge_scaf", "Organelle_hit", "nrRNA_hit", "Contamination_hit", "Note"]
        header_str = printer_list(header_list, sep='\t')
        f.write(header_str + "\n")

        for contig_id in contig_list:
            info_list = [contig_id, genome_fasta[contig_id].len(),
                         genome_depth_dict[contig_id], gc_dict[contig_id], (contig_id in contig_in_huge_scaffold)]
            if contig_id not in subject_contig_dict:
                info_list = [contig_id,
                             genome_fasta[contig_id].len(),
                             "0", "0", str(
                                 contig_id in contig_in_huge_scaffold),
                             genome_depth_dict[contig_id], "False", ""
                             ]
                info_str = printer_list(info_list, sep='\t')
                f.write(info_str + "\n")
            else:
                info_list = [contig_id,
                             subject_contig_dict[contig_id]['contig_length'],
                             subject_contig_dict[contig_id]['subject_range_length'],
                             "%.2f" % subject_contig_dict[contig_id]["contig_ratio"],
                             str(contig_id in contig_in_huge_scaffold),
                             genome_depth_dict[contig_id]
                             ]

                query_info = []
                query_id_list = sorted(subject_contig_dict[contig_id]['query_info'],
                                       key=lambda x: subject_contig_dict[contig_id]['query_info'][x]['query_ratio'],
                                       reverse=True)

                for query_id in query_id_list:
                    query_length = subject_contig_dict[contig_id]['query_info'][query_id]["query_length"]
                    query_range_length = subject_contig_dict[contig_id]['query_info'][query_id][
                        "query_range_length"]
                    query_ratio = subject_contig_dict[contig_id]['query_info'][query_id]["query_ratio"]
                    string_tmp = "%s:%d/%d(%.2f)" % (query_id,
                                                     query_range_length, query_length, query_ratio)
                    query_info.append(string_tmp)

                query_info_str = printer_list(query_info, sep=",")

                if subject_contig_dict[contig_id]['contig_length'] < 1000000 and subject_contig_dict[contig_id][
                        "contig_ratio"] > 0.5 and not (contig_id in contig_in_huge_scaffold) and genome_depth_dict[
                        contig_id] > 100:
                    info_list.append('True')
                else:
                    info_list.append('False')

                info_list.append(query_info_str)

                info_str = printer_list(info_list, sep='\t')

                f.write(info_str + "\n")


if __name__ == "__main__":
    class abc(object):
        pass

    args = abc()

    args.genome_file = "/lustre/home/xuyuxing/Database/Lindenbergia/genome/hic_canu/filter/nextpolish2_canu.final.rename.fasta"
    args.hic_assembly = "/lustre/home/xuyuxing/Database/Lindenbergia/genome/hic_canu/filter/Llv.reviewed.rename.assembly"
    args.genome_seq_depth = "/lustre/home/xuyuxing/Database/Lindenbergia/genome/hic_canu/filter/Llv.GSS.depth"
    args.contamination_results = "/lustre/home/xuyuxing/Database/Lindenbergia/genome/hic_canu/filter/contamination_report.txt"
    args.output_dir = "/lustre/home/xuyuxing/Database/Lindenbergia/genome/hic_canu/filter/Llv"

    ContigFilter_main(args)



import matplotlib.path as mpath
import matplotlib.patches as mpatches

def scale_to_give_size(a_point_coord, target_size, raw_size, blank=(0, 0, 0, 0), reverse_x=False, reverse_y=False):
    """
    scale a point in a raw size to a new size

    point = (0,0)
    raw_size = (-2,2,-2,2) # x leaf,x right, y tail,y head

    target_size = (0,20,0,100)
    new point = (10,50)

    """

    b_x_L, b_x_R, b_y_T, b_y_H = blank
    a_x_L, a_x_R, a_y_T, a_y_H = target_size
    w_x_L, w_x_R, w_y_T, w_y_H = a_x_L + b_x_L, a_x_R - b_x_R, a_y_T + b_y_T, a_y_H - b_y_H
    r_x_L, r_x_R, r_y_T, r_y_H = raw_size

    if reverse_x:
        o_x = w_x_R - (a_point_coord[0] - r_x_L) / (r_x_R - r_x_L) * (w_x_R - w_x_L)
    else:
        o_x = (a_point_coord[0] - r_x_L) / (r_x_R - r_x_L) * (w_x_R - w_x_L) + w_x_L

    if reverse_y:
        o_y = w_y_H - (a_point_coord[1] - r_y_T) / (r_y_H - r_y_T) * (w_y_H - w_y_T)
    else:
        o_y = (a_point_coord[1] - r_y_T) / (r_y_H - r_y_T) * (w_y_H - w_y_T) + w_y_T

    return o_x, o_y

def line_segment_plot(ax, start, end, facecolor='w', lw=2):
    verts = [start, end]
    codes = [mpath.Path.MOVETO, mpath.Path.LINETO]

    path = mpath.Path(verts, codes)
    patch = mpatches.PathPatch(path, facecolor=facecolor, lw=lw)
    ax.add_patch(patch)





from scipy.stats import chi2_contingency, fisher_exact, hypergeom, ks_2samp, norm, shapiro
import numpy as np
import pingouin as pg
from scipy.signal import find_peaks
from scipy import stats, integrate
from lib.common.fileIO import read_list_file
from fitter import Fitter
import math
import random
from scipy.special import factorial


def n50_stats(len_list, index_list=[10, 20, 30, 40, 50, 60, 70, 80, 90, 100]):
    """
    give me a seq length list or a generator
    """
    length_array = np.array(list(len_list))
    length_array.sort(axis=0, kind='mergesort')
    sorted_length_array = length_array[::-1]
    sum_len = sorted_length_array.sum()
    min_len = sorted_length_array.min()
    max_len = sorted_length_array.max()
    count_sum = len(sorted_length_array)
    percent_stats = np.percentile(sorted_length_array, index_list)

    adder_now = 0
    index_stats_dict = {}
    adder_count = 0
    for i in sorted_length_array:
        adder_now = adder_now + i
        adder_count = adder_count + 1
        for j in index_list:
            if adder_now / sum_len >= j / 100 and j not in index_stats_dict:
                # if index_list.index(j) - 1 >= 0:
                #     used_index.append(index_list[index_list.index(j) - 1])
                index_stats_dict[j] = {'count': adder_count, 'base': adder_now, 'length': i}

    return percent_stats, index_stats_dict, count_sum, sum_len, min_len, max_len

# https://en.wikipedia.org/wiki/False_discovery_rate#Benjamini%E2%80%93Hochberg_procedure
# Benjamini–Hochberg procedure
def get_qvalue(pvalue_dir):
    """

    pvalue_dir = {
        "ID_1":0.001,
        "ID_2":0.02,
        "ID_3":0.002,
        "ID_4":0.8,
        "ID_5":0.05,
        "ID_6":0.1,
    }
    :return:
    """

    sorted_id_list = sorted(pvalue_dir, key=lambda x: pvalue_dir[x])
    total_record = len(sorted_id_list)
    num = 1
    qvalue_dir = {}
    for i in sorted_id_list:
        qvalue = pvalue_dir[i] * total_record / num
        qvalue_dir[i] = qvalue
        num = num + 1
    return qvalue_dir


# we use https://github.com/raphaelvallat/pingouin

# t test
def t_test(list1, list2):
    """
    based on pingouin
    see: https://pingouin-stats.org/generated/pingouin.ttest.html#pingouin.ttest
    :return:
    pandas.DataFrame

    'T' : T-value
    'p-val' : p-value
    'dof' : degrees of freedom
    'cohen-d' : Cohen d effect size
    'CI95%' : 95% confidence intervals of the difference in means
    'power' : achieved power of the test ( = 1 - type II error)
    'BF10' : Bayes Factor of the alternative hypothesis

    example:
        output = t_test([1,2,3,4,5,6,7],[7,6,1,4,3,5,6,4])

    p_value = float(output['p-val'])
    """
    list1 = np.array(list1)
    list2 = np.array(list2)
    return pg.ttest(list1, list2)


# chi2 test
def two_categories_chi2(Contingency_table):
    """
    ref:
    https://codingdisciple.com/chi-squared-python.html
    https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.chi2_contingency.html
    https://en.wikipedia.org/wiki/Contingency_table

    :param Contingency_table: https://en.wikipedia.org/wiki/Contingency_table

        if we have:

            A1          A2
        B1  num1        num2        num1+num2
        B2  num3        num4        num3+num4
            num1+num3   num2+num4   num1+num2+num3+num4

        then input Contingency_table will be:

        Contingency_table = [[num1,num2],[num3,num4]]

    :return:
    """

    obs = np.array(Contingency_table)

    chi2_value, p_value, freedom = chi2_contingency(obs)[0:3]

    return chi2_value, p_value, freedom


def fisher_enrichment(Contingency_table):
    """
    ref:
    https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.fisher_exact.html
    https://www.ncbi.nlm.nih.gov/pmc/articles/PMC4743627/
    http://mengnote.blogspot.com/2012/12/calculate-correct-hypergeometric-p.html

    :param Contingency_table: https://en.wikipedia.org/wiki/Contingency_table

        if we have:

            A1          A2
        B1  num1        num2        num1+num2
        B2  num3        num4        num3+num4
            num1+num3   num2+num4   num1+num2+num3+num4

        then input Contingency_table will be:

        Contingency_table = [[num1,num2],[num3,num4]]

    :return:
    """

    oddsratio, pvalue = fisher_exact(Contingency_table)

    return oddsratio, pvalue


def hypergeom_enrichment(Contingency_table):
    """
    ref:
    https://stackoverflow.com/questions/6594840/what-are-equivalents-to-rs-phyper-function-in-python
    http://mengnote.blogspot.com/2012/12/calculate-correct-hypergeometric-p.html
    https://docs.scipy.org/doc/scipy-0.14.0/reference/generated/scipy.stats.hypergeom.html
    Zhou, P., et al. (2019). "Dynamic Patterns of Gene Expression Additivity and Regulatory Variation throughout Maize Development." Mol Plant 12(3): 410-425.

    Models drawing objects from a bin. M is total number of objects, n is total number of Type I objects. RV counts number of Type I objects in N drawn without replacement from population.

    :param Contingency_table: https://en.wikipedia.org/wiki/Contingency_table

        if we have:

            A1          A2
        B1  num1        num2        num1+num2
        B2  num3        num4        num3+num4
            num1+num3   num2+num4   num1+num2+num3+num4

        then input Contingency_table will be:

        Contingency_table = [[num1,num2],[num3,num4]]
    :return:
    """

    # Contingency_table = [[45, 1284], [47, 6234]]

    [[num1, num2], [num3, num4]] = Contingency_table
    x = num1
    M = num1 + num2 + num3 + num4
    n = num1 + num3
    N = num1 + num2

    p_value = hypergeom.sf(x, M, n, N)

    rv = hypergeom(M, n, N)

    # probability mass function
    pmf_tmp = rv.pmf(np.arange(0, n + 1))

    expected_num = list(pmf_tmp).index(max(pmf_tmp)) + 1

    observed_num = x

    return observed_num, expected_num, p_value


# Kolmogorov-Smirnov test
"""
how it wort
http://www.physics.csbsju.edu/stats/KS-test.html

one sample vs hypothesis distribution
https://docs.scipy.org/doc/scipy-0.14.0/reference/generated/scipy.stats.kstest.html

two sample (two distribution) compare
https://docs.scipy.org/doc/scipy-0.14.0/reference/generated/scipy.stats.ks_2samp.html
"""


def two_distribution_test(data1, data2):
    """

    :param data1: list for all data from set 1
    :param data2: list for all data from set 2
    :return: KS statistic (D statistic) and p-value

    If the K-S statistic is small or the p-value is high, then we cannot reject the hypothesis that the distributions of the two samples are the same.


    https://stats.idre.ucla.edu/stata/faq/how-can-i-test-for-equality-of-distribution/
    HOW CAN I TEST FOR EQUALITY OF DISTRIBUTION?
    An alternative test to the classic t-test is the Kolmogorov-Smirnov test for equality of distribution functions.
    In a simple example, we’ll see if the distribution of writing test scores across gender are equal using the High-School
    and Beyond 2000 data set. We’ll first do a kernel density plot of writing scores by gender.

    https://docs.scipy.org/doc/scipy-0.14.0/reference/generated/scipy.stats.ks_2samp.html

    file_list = [
    '/lustre/home/xuyuxing/Liu/A_D_Arabidopsis_F.txt',
    '/lustre/home/xuyuxing/Liu/A_D_Arabidopsis_O.txt',
    '/lustre/home/xuyuxing/Liu/A_D_Dodder_F.txt',
    '/lustre/home/xuyuxing/Liu/A_D_Dodder_O.txt',
    '/lustre/home/xuyuxing/Liu/S_D_Dodder_F.txt',
    '/lustre/home/xuyuxing/Liu/S_D_Dodder_O.txt',
    '/lustre/home/xuyuxing/Liu/S_D_Soybean_F.txt',
    '/lustre/home/xuyuxing/Liu/S_D_Soybean_O.txt',
    ]

    from lib.common.fileIO import read_list_file

    data_list = []
    for i in file_list:
        data_list.append([float(j) for j in read_list_file(i) if j != ''])

    for i in [0,2,4,6]:
        print(two_distribution_test(data_list[i],data_list[i+1]))

    """

    data1 = np.array(data1)
    data2 = np.array(data2)

    return tuple(ks_2samp(data1, data2))


# anova
# https://www.analyticsvidhya.com/blog/2018/01/anova-analysis-of-variance/
# https://pingouin-stats.org/generated/pingouin.anova.html#pingouin.anova

def anova(input_dataframe, dv, between, ss_type=2, detailed=True):
    """
    see https://pingouin-stats.org/generated/pingouin.anova.html#pingouin.anova for detail

    aov = pg.anova(dv='len', between=['supp', 'dose'], data=input_dataframe, detailed=True)

    input_dataframe should be pandas dataframe, each column is a variable, first row is name of variable for the column
    if in a csv file, it should like 'https://vincentarelbundock.github.io/Rdatasets/csv/datasets/ToothGrowth.csv'
    make df as:
    data = 'https://vincentarelbundock.github.io/Rdatasets/csv/datasets/ToothGrowth.csv'
    df = pd.read_csv(data, index_col=0)

    dv is one of the variable name which as y

    between is list of variable name

    :return
    'Source' : Factor names
    'SS' : Sums of squares
    'DF' : Degrees of freedom
    'MS' : Mean squares
    'F' : F-values
    'p-unc' : uncorrected p-values
    'np2' : Partial eta-square effect sizes

    """
    aov = pg.anova(dv=dv, between=between, data=input_dataframe, detailed=detailed, ss_type=ss_type)
    return aov


def norm_test(data_list):
    if len(data_list) > 3000:
        data_list_random = random.sample(data_list, 3000)
        statistic, pvalue = stats.shapiro(data_list_random)
    else:
        statistic, pvalue = stats.shapiro(data_list)
    
    return statistic, pvalue


def get_threshold(data_list, plot_flag=False, p_value=0.05):
    x, bins = np.histogram(data_list, bins='auto')
    peaks, properties = find_peaks(x, prominence=1, width=1)

    peak_dict = {}
    for i in range(len(peaks)):
        peak_dict[i] = (peaks[i], properties['widths'][i], properties['prominences'][i])

    top_peak = sorted(peak_dict, key=lambda x:peak_dict[x][2], reverse=True)[0]

    # if plot_flag:
    #     plt.plot(x)
    #     plt.plot(peaks[top_peak], x[peaks[top_peak]], "x")
    #     plt.vlines(x=peaks[top_peak], ymin=x[peaks[top_peak]] - properties["prominences"][top_peak],
    #             ymax = x[peaks[top_peak]], color = "C1")
    #     plt.hlines(y=properties["width_heights"][top_peak], xmin=properties["left_ips"][top_peak],
    #             xmax=properties["right_ips"][top_peak], color = "C1")
    #     plt.show()

    peak_site = peak_dict[top_peak][0]
    peak_width = peak_dict[top_peak][1]
    top_peak_range_index = int(peak_site-peak_width), math.ceil(peak_site+peak_width)
    top_peak_range = [bins[i] for i in top_peak_range_index]

    peak_data_list = [i for i in data_list if i >= top_peak_range[0] and i <= top_peak_range[1]]

    statistic, pvalue = norm_test(peak_data_list)

    f = Fitter(peak_data_list, distributions=['norm'])
    f.fit()

    # if plot_flag:
    #     f.summary()

    mu, sigma = f.fitted_param['norm']

    threshold = norm.ppf(p_value, loc=mu, scale = sigma)

    return threshold, mu, sigma, statistic, pvalue

# curve fit by gaussian distribution
def gaussian_funtion(x, *params):
    """
    y = $a * exp\biggl(-\frac{(x-b)^2}{c^2}\biggr)$
    """
    num_func = int(len(params)/3)

    y_list = []
    for i in range(num_func):
        y = np.zeros_like(x)
        param_range = list(range(3*i,3*(i+1),1))
        amp = params[int(param_range[0])]
        ctr = params[int(param_range[1])]
        wid = params[int(param_range[2])]
        y = y + amp * np.exp( -((x - ctr)/wid)**2)
        y_list.append(y)

    y_sum = np.zeros_like(x)
    for i in y_list:
        y_sum = y_sum + i

    y_sum = y_sum + params[-1]

    return y_sum    

def fit_plot(x, *params):
    num_func = int(len(params)/3)
    y_list = []
    for i in range(num_func):
        y = np.zeros_like(x)
        param_range = list(range(3*i,3*(i+1),1))
        amp = params[int(param_range[0])]
        ctr = params[int(param_range[1])]
        wid = params[int(param_range[2])]
        y = y + amp * np.exp( -((x - ctr)/wid)**2) + params[-1]
        y_list.append(y)
    return y_list

    
def get_peak_area(*params):
    # params = (1273,65,20,0)
    ctr = params[1]
    wid = params[2]
    area = integrate.quad(lambda x:gaussian_funtion(x, *params), ctr-6*wid, ctr+6*wid)
    return area[0]

"""
from scipy.optimize import curve_fit

How to use curve fit by gaussian distribution

有三个要优化的参数：a(amp, 峰高)，b(ctr, 位置)和c(wid, 峰宽)。

假设有两个峰重叠进去了
guess = []
guess.append([300, 0.8, 0.5])
guess.append([300, 1.2, 0.5])

background = 5

guess_total = []
for i in guess:
    guess_total.extend(i)
guess_total.append(background)

popt, pcov = curve_fit(func, x, y, p0=guess_total)

fit = func(x, *popt)
plt.scatter(x, y, s=20)
plt.plot(x, fit , ls='-', c='black', lw=1)

y_list = fit_plot(x, *popt)
baseline = np.zeros_like(x) + popt[-1]
for n,i in enumerate(y_list):
    plt.fill_between(x, i, baseline, facecolor=cm.rainbow(n/len(y_list)), alpha=0.6)
"""

def poisson(k, lamb):
  return (lamb**k/factorial(k)) * np.exp(-lamb)


import math


def base_translate(num, base_num):
    if num < base_num:
        return [num]
    else:
        n = int(math.log10(num) / math.log10(base_num)) + 1
        output_list = []
        for i in range(n):
            num_add = num // (base_num ** (n - i - 1))
            output_list.append(num_add)
            num = num - num_add * (base_num ** (n - i - 1))
        return output_list


# coding:utf-8

import matplotlib

matplotlib.use('Agg')

import matplotlib.pyplot as plt
from matplotlib.ticker import MultipleLocator
import numpy as np

t = range(-100, 100)
s = abs(np.random.randn(len(t)))
fig = plt.gcf()
relative_site = 500
transcript_id = "test"

fig.set_size_inches(10.5, 10.5)

plt.subplot(611)
plt.axis([min(t), max(t), min(s), max(s)])
plt.bar(t, s, width=1)
# plt.xlabel('time (s)')
plt.ylabel('Wild type cmc+')
plt.title('Average_Peak = %.5f' % 1.11111111)
ax = plt.gca()
xmajorLocator = MultipleLocator(10)
ax.xaxis.set_major_locator(xmajorLocator)
ax.spines['left'].set_visible(False)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
plt.yticks([])
plt.xticks([])
plt.grid(False)
line = ax.axvline(x=0, color='r', linewidth=1, linestyle="--")

plt.subplot(612)
plt.axis([min(t), max(t), min(s), max(s)])
plt.bar(t, s, width=1)
# plt.xlabel('time (s)')
plt.ylabel('Wild type cmc-')
plt.title(u'Ψ ' + str(relative_site))
ax = plt.gca()
xmajorLocator = MultipleLocator(10)
ax.xaxis.set_major_locator(xmajorLocator)
ax.spines['left'].set_visible(False)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
plt.yticks([])
plt.xticks([])
plt.grid(False)
line = ax.axvline(x=0, color='r', linewidth=1, linestyle="--")

plt.subplot(613)
plt.title(transcript_id + " " + str(relative_site + min(t)) + "-" + str(relative_site + max(t)))
plt.yticks([])
plt.xticks([])
ax = plt.gca()
ax.spines['left'].set_visible(False)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.spines['bottom'].set_visible(False)
plt.grid(False)

plt.subplot(614)
plt.axis([min(t), max(t), min(s), max(s)])
plt.bar(t, s, width=1)
# plt.xlabel('time (s)')
plt.ylabel('Mutant type cmc+')
plt.title('Average_Peak = %.5f' % 1.111111111)
ax = plt.gca()
xmajorLocator = MultipleLocator(10)
ax.xaxis.set_major_locator(xmajorLocator)
ax.spines['left'].set_visible(False)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
plt.yticks([])
plt.xticks([])
plt.grid(False)
line = ax.axvline(x=0, color='r', linewidth=1, linestyle="--")

plt.subplot(615)
plt.axis([min(t), max(t), min(s), max(s)])
plt.bar(t, s, width=1)
# plt.xlabel('time (s)')
plt.ylabel('Mutant type cmc-')
plt.title(u'Ψ ' + str(relative_site))
ax = plt.gca()
xmajorLocator = MultipleLocator(10)
ax.xaxis.set_major_locator(xmajorLocator)
ax.spines['left'].set_visible(False)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
plt.yticks([])
plt.xticks([])
plt.grid(False)
line = ax.axvline(x=0, color='r', linewidth=1, linestyle="--")

plt.subplot(616)
plt.title(transcript_id + " " + str(relative_site + min(t)) + "-" + str(relative_site + max(t)))
plt.yticks([])
plt.xticks([])
ax = plt.gca()
ax.spines['left'].set_visible(False)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
ax.spines['bottom'].set_visible(False)
plt.grid(False)
plt.subplots_adjust(hspace=0.5)
plt.savefig(transcript_id + "_" + str(relative_site) + '.pdf', dpi=1000)

plt.show()
